#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.common import AppArmorBug, AppArmorException, hasher
from apparmor.logparser import ReadLog
from apparmor.rule.ptrace import PtraceRule, PtraceRuleset
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops

_ = init_translation()

exp = namedtuple(
    'exp', ('audit', 'allow_keyword', 'deny', 'comment', 'access', 'all_access', 'peer',
            'all_peers'),
)


# # --- tests for single PtraceRule --- #

class PtraceTest(AATest):
    def _compare_obj(self, obj, expected):
        self.assertEqual(expected.allow_keyword, obj.allow_keyword)
        self.assertEqual(expected.audit, obj.audit)
        self.assertEqual(expected.access, obj.access)
        if obj.peer:
            self.assertEqual(expected.peer, obj.peer.regex)
        else:
            self.assertEqual(expected.peer, obj.peer)
        self.assertEqual(expected.all_access, obj.all_access)
        self.assertEqual(expected.all_peers, obj.all_peers)
        self.assertEqual(expected.deny, obj.deny)
        self.assertEqual(expected.comment, obj.comment)


class PtraceTestParse(PtraceTest):
    tests = (
        # PtraceRule object            audit  allow  deny   comment       access               all?   peer           all?
        ('ptrace,',                   exp(False, False, False, '',       None,                 True,  None,         True)),
        # ('ptrace (),',                exp(False, False, False, '',       None,                 True,  None,         True)),  # XXX also broken in SignalRule?
        ('ptrace read,',              exp(False, False, False, '',       {'read'},             False, None,         True)),
        ('ptrace (read, tracedby),',  exp(False, False, False, '',       {'read', 'tracedby'}, False, None,         True)),
        ('ptrace read,',              exp(False, False, False, '',       {'read'},             False, None,         True)),
        ('deny ptrace read, # cmt',   exp(False, False, True,  ' # cmt', {'read'},             False, None,         True)),
        ('audit allow ptrace,',       exp(True,  True,  False, '',       None,                 True,  None,         True)),
        ('ptrace peer=unconfined,',   exp(False, False, False, '',       None,                 True,  'unconfined', False)),
        ('ptrace peer="unconfined",', exp(False, False, False, '',       None,                 True,  'unconfined', False)),
        ('ptrace read,',              exp(False, False, False, '',       {'read'},             False, None,         True)),
        ('ptrace peer=/foo,',         exp(False, False, False, '',       None,                 True,  '/foo',       False)),
        ('ptrace r peer=/foo,',       exp(False, False, False, '',       {'r'},                False, '/foo',       False)),
        ('ptrace r peer="/foo bar",', exp(False, False, False, '',       {'r'},                False, '/foo bar',   False)),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(PtraceRule.match(rawrule))
        obj = PtraceRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class PtraceTestParseInvalid(PtraceTest):
    tests = (
        ('ptrace foo,',           AppArmorException),
        ('ptrace foo bar,',       AppArmorException),
        ('ptrace foo int,',       AppArmorException),
        ('ptrace read bar,',      AppArmorException),
        ('ptrace read tracedby,', AppArmorException),
        ('ptrace peer=,',         AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(PtraceRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            PtraceRule.create_instance(rawrule)


class PtraceTestParseFromLog(PtraceTest):
    def test_ptrace_from_log(self):
        parser = ReadLog('', '', '')
        event = 'type=AVC msg=audit(1409700683.304:547661): apparmor="DENIED" operation="ptrace" profile="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace" pid=22465 comm="ptrace" requested_mask="tracedby" denied_mask="tracedby" peer="/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace"'

        parsed_event = parser.parse_event(event)

        self.assertEqual(parsed_event, {
            'request_mask': 'tracedby',
            'denied_mask': 'tracedby',
            'error_code': 0,
            'magic_token': 0,
            'parent': 0,
            'profile': '/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace',
            'peer': '/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace',
            'operation': 'ptrace',
            'resource': None,
            'info': None,
            'aamode': 'REJECTING',
            'time': 1409700683,
            'active_hat': None,
            'pid': 22465,
            'task': 0,
            'attr': None,
            'name2': None,
            'name': None,
            'family': None,
            'protocol': None,
            'sock_type': None,
            'class': None,
        })

        obj = PtraceRule(parsed_event['denied_mask'], parsed_event['peer'], log_event=parsed_event)

        #             audit  allow  deny  comment  access     all?   peer                                                          all?
        expected = exp(False, False, False, '', {'tracedby'}, False, '/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace', False)

        self._compare_obj(obj, expected)

        self.assertEqual(
            obj.get_raw(1),
            '  ptrace tracedby peer=/home/ubuntu/bzr/apparmor/tests/regression/apparmor/ptrace,')

    def test_null_ptrace_from(self):
        log = 'type=AVC msg=audit(1495217772.047:4471): apparmor="DENIED" operation="ptrace" profile="/usr/bin/pidgin" pid=21704 comm="pidgin" peer="//null-"'
        parser = ReadLog('', '', '')

        hl = hasher()

        ev = parser.parse_event(log)
        PtraceRule.hashlog_from_event(hl, ev)

        expected = {'//null-': {None: True}}
        self.assertEqual(hl, expected)

        sr = PtraceRule.from_hashlog(hl)

        with self.assertRaises(StopIteration):
            next(sr)


class PtraceFromInit(PtraceTest):
    tests = (
        # PtraceRule object                              audit  allow  deny   comment  access         all?   peer          all?
        (PtraceRule('r', 'unconfined', deny=True),   exp(False, False, True,  '',      {'r'},         False, 'unconfined', False)),
        (PtraceRule(('r', 'read'), '/bin/foo'),      exp(False, False, False, '',      {'r', 'read'}, False, '/bin/foo',   False)),
        (PtraceRule(PtraceRule.ALL, '/bin/foo'),     exp(False, False, False, '',      None,          True,  '/bin/foo',   False)),
        (PtraceRule('rw', '/bin/foo'),               exp(False, False, False, '',      {'rw'},        False, '/bin/foo',   False)),
        (PtraceRule('rw', PtraceRule.ALL),           exp(False, False, False, '',      {'rw'},        False, None,         True)),
        (PtraceRule(PtraceRule.ALL, PtraceRule.ALL), exp(False, False, False, '',      None,          True,  None,         True)),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidPtraceInit(AATest):
    tests = (
        # (init params, expected exception)
        (('',     '/foo'), AppArmorBug),        # empty access
        (('read', ''),     AppArmorBug),        # empty peer
        (('    ', '/foo'), AppArmorBug),        # whitespace access
        (('read', '    '), AppArmorBug),        # whitespace peer
        (('xyxy', '/foo'), AppArmorException),  # invalid access
        # XXX is 'invalid peer' possible at all?
        ((dict(), '/foo'), AppArmorBug),        # wrong type for access
        ((None,   '/foo'), AppArmorBug),        # wrong type for access
        (('read', dict()), AppArmorBug),        # wrong type for peer
        (('read', None),   AppArmorBug),        # wrong type for peer
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            PtraceRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            PtraceRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            PtraceRule('r')


class InvalidPtraceTest(AATest):
    def _check_invalid_rawrule(self, rawrule):
        obj = None
        self.assertFalse(PtraceRule.match(rawrule))
        with self.assertRaises(AppArmorException):
            obj = PtraceRule.create_instance(rawrule)

        self.assertIsNone(obj, 'PtraceRule handed back an object unexpectedly')

    def test_invalid_ptrace_missing_comma(self):
        self._check_invalid_rawrule('ptrace')  # missing comma

    def test_invalid_non_PtraceRule(self):
        self._check_invalid_rawrule('dbus,')  # not a ptrace rule

    def test_empty_data_1(self):
        obj = PtraceRule('read', '/foo')
        obj.access = ''
        # no access set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)

    def test_empty_data_2(self):
        obj = PtraceRule('read', '/foo')
        obj.peer = ''
        # no ptrace set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)


class WritePtraceTestAATest(AATest):
    def _run_test(self, rawrule, expected):
        self.assertTrue(PtraceRule.match(rawrule))
        obj = PtraceRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    tests = (
        #  raw rule                                           clean rule
        ('ptrace,',                                           'ptrace,'),
        ('     ptrace         ,    # foo        ',            'ptrace, # foo'),
        ('    audit     ptrace read,',                        'audit ptrace read,'),
        ('    audit     ptrace (read  ),',                    'audit ptrace read,'),
        ('    audit     ptrace (read , tracedby ),',          'audit ptrace (read tracedby),'),
        ('   deny ptrace         read      ,# foo bar',       'deny ptrace read, # foo bar'),
        ('   deny ptrace      (  read      ),  ',             'deny ptrace read,'),
        ('   allow ptrace                        ,# foo bar', 'allow ptrace, # foo bar'),
        ('ptrace,',                                           'ptrace,'),
        ('ptrace (trace),',                                   'ptrace trace,'),
        ('ptrace (tracedby),',                                'ptrace tracedby,'),
        ('ptrace (read),',                                    'ptrace read,'),
        ('ptrace (readby),',                                  'ptrace readby,'),
        ('ptrace (trace read),',                              'ptrace (read trace),'),
        ('ptrace (read tracedby),',                           'ptrace (read tracedby),'),
        ('ptrace r,',                                         'ptrace r,'),
        ('ptrace w,',                                         'ptrace w,'),
        ('ptrace rw,',                                        'ptrace rw,'),
        ('ptrace read,',                                      'ptrace read,'),
        ('ptrace (tracedby),',                                'ptrace tracedby,'),
        ('ptrace w,',                                         'ptrace w,'),
        ('ptrace read peer=foo,',                             'ptrace read peer=foo,'),
        ('ptrace tracedby peer=foo,',                         'ptrace tracedby peer=foo,'),
        ('ptrace (read tracedby) peer=/usr/bin/bar,',         'ptrace (read tracedby) peer=/usr/bin/bar,'),
        ('ptrace (trace read) peer=/usr/bin/bar,',            'ptrace (read trace) peer=/usr/bin/bar,'),
        ('ptrace wr peer=/sbin/baz,',                         'ptrace wr peer=/sbin/baz,'),
        ('priority = 010 deny ptrace      (  read      ),  ', 'priority=10 deny ptrace read,'),
        ('priority = 0   deny ptrace      (  read      ),  ', 'priority=0 deny ptrace read,'),
        ('priority = -21 deny ptrace      (  read      ),  ', 'priority=-21 deny ptrace read,'),
        ('priority = +83 deny ptrace      (  read      ),  ', 'priority=83 deny ptrace read,'),
    )

    def test_write_manually(self):
        obj = PtraceRule('read', '/foo', allow_keyword=True)

        expected = '    allow ptrace read peer=/foo,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class PtraceCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = PtraceRule.create_instance(self.rule)
        check_obj = PtraceRule.create_instance(param)

        self.assertTrue(PtraceRule.match(param))

        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class PtraceCoveredTest_01(PtraceCoveredTest):
    rule = 'ptrace read,'

    tests = (
        #   rule                          equal  strict equal  covered  covered exact
        ('ptrace,',                      (False, False,        False,   False)),
        ('ptrace read,',                 (True,  True,         True,    True)),
        ('ptrace read peer=unconfined,', (False, False,        True,    True)),
        ('ptrace read, # comment',       (True,  False,        True,    True)),
        ('allow ptrace read,',           (True,  False,        True,    True)),
        ('ptrace     read,',             (True,  False,        True,    True)),
        ('audit ptrace read,',           (False, False,        False,   False)),
        ('audit ptrace,',                (False, False,        False,   False)),
        ('ptrace tracedby,',             (False, False,        False,   False)),
        ('audit deny ptrace read,',      (False, False,        False,   False)),
        ('deny ptrace read,',            (False, False,        False,   False)),
    )


class PtraceCoveredTest_02(PtraceCoveredTest):
    rule = 'audit ptrace read,'

    tests = (
        #   rule                equal  strict equal  covered  covered exact
        ('      ptrace read,', (False, False,        True,    False)),
        ('audit ptrace read,', (True,  True,         True,    True)),
        ('      ptrace,',      (False, False,        False,   False)),
        ('audit ptrace,',      (False, False,        False,   False)),
        ('ptrace tracedby,',   (False, False,        False,   False)),
    )


class PtraceCoveredTest_03(PtraceCoveredTest):
    rule = 'ptrace,'

    tests = (
        #   rule                equal  strict equal  covered  covered exact
        ('      ptrace,',      (True,  True,         True,    True)),
        ('allow ptrace,',      (True,  False,        True,    True)),
        ('      ptrace read,', (False, False,        True,    True)),
        ('      ptrace w,',    (False, False,        True,    True)),
        ('audit ptrace,',      (False, False,        False,   False)),
        ('deny  ptrace,',      (False, False,        False,   False)),
    )


class PtraceCoveredTest_04(PtraceCoveredTest):
    rule = 'deny ptrace read,'

    tests = (
        #   rule                         equal  strict equal  covered   covered exact
        ('      deny ptrace read,',     (True,  True,         True,    True)),
        ('audit deny ptrace read,',     (False, False,        False,   False)),
        ('           ptrace read,',     (False, False,        False,   False)),  # XXX should covered be true here?
        ('      deny ptrace tracedby,', (False, False,        False,   False)),
        ('      deny ptrace,',          (False, False,        False,   False)),
    )


class PtraceCoveredTest_05(PtraceCoveredTest):
    rule = 'ptrace read peer=unconfined,'

    tests = (
        #   rule                                equal  strict equal  covered  covered exact
        ('ptrace,',                            (False, False,        False,   False)),
        ('ptrace read,',                       (False, False,        False,   False)),
        ('ptrace read peer=unconfined,',       (True,  True,         True,    True)),
        ('ptrace peer=unconfined,',            (False, False,        False,   False)),
        ('ptrace read, # comment',             (False, False,        False,   False)),
        ('allow ptrace read,',                 (False, False,        False,   False)),
        ('allow ptrace read peer=unconfined,', (True,  False,        True,    True)),
        ('allow ptrace read peer=/foo/bar,',   (False, False,        False,   False)),
        ('allow ptrace read peer=/**,',        (False, False,        False,   False)),
        ('allow ptrace read peer=**,',         (False, False,        False,   False)),
        ('ptrace    read,',                    (False, False,        False,   False)),
        ('ptrace    read peer=unconfined,',    (True,  False,        True,    True)),
        ('audit ptrace read peer=unconfined,', (False, False,        False,   False)),
        ('audit ptrace,',                      (False, False,        False,   False)),
        ('ptrace tracedby,',                   (False, False,        False,   False)),
        ('audit deny ptrace read,',            (False, False,        False,   False)),
        ('deny ptrace read,',                  (False, False,        False,   False)),
    )


class PtraceCoveredTest_06(PtraceCoveredTest):
    rule = 'ptrace read peer=/foo/bar,'

    tests = (
        #   rule                             equal   strict equal  covered  covered exact
        ('ptrace,',                          (False, False,        False,   False)),
        ('ptrace read,',                     (False, False,        False,   False)),
        ('ptrace read peer=/foo/bar,',       (True,  True,         True,    True)),
        ('ptrace read peer=/foo/*,',         (False, False,        False,   False)),
        ('ptrace read peer=/**,',            (False, False,        False,   False)),
        ('ptrace read peer=/what/*,',        (False, False,        False,   False)),
        ('ptrace peer=/foo/bar,',            (False, False,        False,   False)),
        ('ptrace read, # comment',           (False, False,        False,   False)),
        ('allow ptrace read,',               (False, False,        False,   False)),
        ('allow ptrace read peer=/foo/bar,', (True,  False,        True,    True)),
        ('ptrace    read,',                  (False, False,        False,   False)),
        ('ptrace    read peer=/foo/bar,',    (True,  False,        True,    True)),
        ('ptrace    read peer=/what/ever,',  (False, False,        False,   False)),
        ('audit ptrace read peer=/foo/bar,', (False, False,        False,   False)),
        ('audit ptrace,',                    (False, False,        False,   False)),
        ('ptrace tracedby,',                 (False, False,        False,   False)),
        ('audit deny ptrace read,',          (False, False,        False,   False)),
        ('deny ptrace read,',                (False, False,        False,   False)),
    )


class PtraceCoveredTest_07(PtraceCoveredTest):
    rule = 'ptrace read peer=**,'

    tests = (
        #   rule                              equal  strict equal  covered  covered exact
        ('ptrace,',                          (False, False,        False,   False)),
        ('ptrace read,',                     (False, False,        False,   False)),
        ('ptrace read peer=/foo/bar,',       (False, False,        True,    True)),
        ('ptrace read peer=/foo/*,',         (False, False,        False,   False)),  # TODO: wildcard vs. wildcard never matches in is_covered_aare()
        ('ptrace read peer=/**,',            (False, False,        False,   False)),  # TODO: wildcard vs. wildcard never matches in is_covered_aare()
        ('ptrace read peer=/what/*,',        (False, False,        False,   False)),  # TODO: wildcard vs. wildcard never matches in is_covered_aare()
        ('ptrace peer=/foo/bar,',            (False, False,        False,   False)),
        ('ptrace read, # comment',           (False, False,        False,   False)),
        ('allow ptrace read,',               (False, False,        False,   False)),
        ('allow ptrace read peer=/foo/bar,', (False, False,        True,    True)),
        ('ptrace    read,',                  (False, False,        False,   False)),
        ('ptrace    read peer=/foo/bar,',    (False, False,        True,    True)),
        ('ptrace    read peer=/what/ever,',  (False, False,        True,    True)),
        ('audit ptrace read peer=/foo/bar,', (False, False,        False,   False)),
        ('audit ptrace,',                    (False, False,        False,   False)),
        ('ptrace tracedby,',                 (False, False,        False,   False)),
        ('audit deny ptrace read,',          (False, False,        False,   False)),
        ('deny ptrace read,',                (False, False,        False,   False)),
    )


class PtraceCoveredTest_08(PtraceCoveredTest):
    rule = 'ptrace (trace, tracedby) peer=/foo/*,'

    tests = (
        #   rule                                    equal  strict equal  covered  covered exact
        ('ptrace,',                                (False, False,        False,  False)),
        ('ptrace trace,',                          (False, False,        False,  False)),
        ('ptrace (tracedby, trace),',              (False, False,        False,  False)),
        ('ptrace trace peer=/foo/bar,',            (False, False,        True,   True)),
        ('ptrace (tracedby trace) peer=/foo/bar,', (False, False,        True,   True)),
        ('ptrace (tracedby, trace) peer=/foo/*,',  (True,  False,        True,   True)),
        ('ptrace tracedby peer=/foo/bar,',         (False, False,        True,   True)),
        ('ptrace trace peer=/foo/*,',              (False, False,        True,   True)),
        ('ptrace trace peer=/**,',                 (False, False,        False,  False)),
        ('ptrace trace peer=/what/*,',             (False, False,        False,  False)),
        ('ptrace peer=/foo/bar,',                  (False, False,        False,  False)),
        ('ptrace trace, # comment',                (False, False,        False,  False)),
        ('allow ptrace trace,',                    (False, False,        False,  False)),
        ('allow ptrace trace peer=/foo/bar,',      (False, False,        True,   True)),
        ('ptrace    trace,',                       (False, False,        False,  False)),
        ('ptrace    trace peer=/foo/bar,',         (False, False,        True,   True)),
        ('ptrace    trace peer=/what/ever,',       (False, False,        False,  False)),
        ('audit ptrace trace peer=/foo/bar,',      (False, False,        False,  False)),
        ('audit ptrace,',                          (False, False,        False,  False)),
        ('ptrace tracedby,',                       (False, False,        False,  False)),
        ('audit deny ptrace trace,',               (False, False,        False,  False)),
        ('deny ptrace trace,',                     (False, False,        False,  False)),
    )


class PtraceCoveredTest_Invalid(AATest):
    def test_borked_obj_is_covered_1(self):
        obj = PtraceRule.create_instance('ptrace read peer=/foo,')

        testobj = PtraceRule('read', '/foo')
        testobj.access = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_borked_obj_is_covered_2(self):
        obj = PtraceRule.create_instance('ptrace read peer=/foo,')

        testobj = PtraceRule('read', '/foo')
        testobj.peer = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_covered(self):
        raw_rule = 'ptrace read,'

        class SomeOtherClass(PtraceRule):
            pass

        obj = PtraceRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal_1(self):
        raw_rule = 'ptrace read,'

        class SomeOtherClass(PtraceRule):
            pass

        obj = PtraceRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)

    def test_invalid_is_equal_2(self):
        obj = PtraceRule.create_instance('ptrace read,')

        testobj = PtraceRule.create_instance('ptrace read,')
        testobj.all_peers = False  # make testobj invalid (should trigger exception in _is_equal_aare())

        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class PtraceLogprofHeaderTest(AATest):
    tests = (
        ('ptrace,',                            [                              _('Access mode'), _('ALL'),        _('Peer'), _('ALL')]),  # noqa: E201
        ('ptrace read,',                       [                              _('Access mode'), 'read',          _('Peer'), _('ALL')]),  # noqa: E201
        ('deny ptrace,',                       [_('Qualifier'), 'deny',       _('Access mode'), _('ALL'),        _('Peer'), _('ALL')]),  # noqa: E201
        ('allow ptrace read,',                 [_('Qualifier'), 'allow',      _('Access mode'), 'read',          _('Peer'), _('ALL')]),
        ('audit ptrace read,',                 [_('Qualifier'), 'audit',      _('Access mode'), 'read',          _('Peer'), _('ALL')]),
        ('audit deny ptrace read,',            [_('Qualifier'), 'audit deny', _('Access mode'), 'read',          _('Peer'), _('ALL')]),
        ('ptrace (read, tracedby) peer=/foo,', [                              _('Access mode'), 'read tracedby', _('Peer'), '/foo']),  # noqa: E201
    )

    def _run_test(self, params, expected):
        obj = PtraceRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


# --- tests for PtraceRuleset --- #

class PtraceRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = PtraceRuleset()
        ruleset_2 = PtraceRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

        # test __repr__() for empty ruleset
        self.assertEqual(str(ruleset), '<PtraceRuleset (empty) />')

    def test_ruleset_1(self):
        ruleset = PtraceRuleset()
        rules = (
            'ptrace peer=/foo,',
            'ptrace read,',
        )

        expected_raw = [
            'ptrace peer=/foo,',
            'ptrace read,',
            '',
        ]

        expected_clean = [
            'ptrace peer=/foo,',
            'ptrace read,',
            '',
        ]

        for rule in rules:
            ruleset.add(PtraceRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())

        # test __repr__() for non-empty ruleset
        self.assertEqual(
            str(ruleset), '<PtraceRuleset>\n  ptrace peer=/foo,\n  ptrace read,\n</PtraceRuleset>')

    def test_ruleset_2(self):
        ruleset = PtraceRuleset()
        rules = (
            'ptrace read peer=/foo,',
            'allow ptrace read,',
            'deny ptrace peer=/bar, # example comment',
        )

        expected_raw = [
            '  ptrace read peer=/foo,',
            '  allow ptrace read,',
            '  deny ptrace peer=/bar, # example comment',
            '',
        ]

        expected_clean = [
            '  deny ptrace peer=/bar, # example comment',
            '',
            '  allow ptrace read,',
            '  ptrace read peer=/foo,',
            '',
        ]

        for rule in rules:
            ruleset.add(PtraceRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw(1))
        self.assertEqual(expected_clean, ruleset.get_clean(1))


class PtraceGlobTestAATest(AATest):
    def setUp(self):
        self.maxDiff = None
        self.ruleset = PtraceRuleset()

    def test_glob_1(self):
        self.assertEqual(self.ruleset.get_glob('ptrace read,'), 'ptrace,')

    # not supported or used yet
    # def test_glob_2(self):
    #     self.assertEqual(self.ruleset.get_glob('ptrace read raw,'), 'ptrace read,')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for ptrace rules
            self.ruleset.get_glob_ext('ptrace read peer=/foo,')


# class PtraceDeleteTestAATest(AATest):
#     pass


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2020 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.common import AppArmorBug, AppArmorException
from apparmor.rule.alias import AliasRule, AliasRuleset
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops

_ = init_translation()

exp = namedtuple('exp', ('comment', 'orig_path', 'target'))

# --- tests for single AliasRule --- #


class AliasTest(AATest):
    def _compare_obj(self, obj, expected):
        # aliases don't support the allow, audit, deny, or priority keyword
        self.assertEqual(False, obj.allow_keyword)
        self.assertEqual(False, obj.audit)
        self.assertEqual(False, obj.deny)
        self.assertEqual(None, obj.priority)

        self.assertEqual(expected.orig_path, obj.orig_path)
        self.assertEqual(expected.target, obj.target)
        self.assertEqual(expected.comment, obj.comment)


class AliasTestParse(AliasTest):
    tests = (
        # rawrule                                         comment       orig_path  target
        ('alias /foo -> /bar,',                       exp('',           '/foo',    '/bar')),
        ('  alias   /foo    ->    /bar ,  # comment', exp(' # comment', '/foo',    '/bar')),
        ('alias "/foo 2" -> "/bar 2"  ,',             exp('',           '/foo 2',  '/bar 2')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(AliasRule.match(rawrule))
        obj = AliasRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class AliasTestParseInvalid(AliasTest):
    tests = (
        # rawrule                 matches regex  exception
        ('alias  ,',              (False,        AppArmorException)),
        ('alias   /foo  ,',       (False,        AppArmorException)),
        ('alias   /foo   ->   ,', (True,         AppArmorException)),
        ('alias   ->   /bar  ,',  (True,         AppArmorException)),
        ('/foo  ->   bar ,',      (False,        AppArmorException)),
    )

    def _run_test(self, rawrule, expected):
        self.assertEqual(AliasRule.match(rawrule), expected[0])
        with self.assertRaises(expected[1]):
            AliasRule.create_instance(rawrule)


class AliasFromInit(AliasTest):
    tests = (
        # AliasRule object                                comment  orig_path  target
        (AliasRule('/foo',  '/bar'),                  exp('',      '/foo',    '/bar')),
        (AliasRule('/foo',  '/bar', comment='# cmt'), exp('# cmt', '/foo',    '/bar')),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidAliasInit(AATest):
    tests = (
        # init params      expected exception
        ((None,   '/bar'), AppArmorBug),        # orig_path not a str
        (('',     '/bar'), AppArmorException),  # empty orig_path
        (('foo',  '/bar'), AppArmorException),  # orig_path not starting with /

        (('/foo', None),   AppArmorBug),        # target not a str
        (('/foo', ''),     AppArmorException),  # empty target
        (('/foo', 'bar'),  AppArmorException),  # target not starting with /
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            AliasRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            AliasRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            AliasRule('/foo')

    def test_invalid_audit(self):
        with self.assertRaises(AppArmorBug):
            AliasRule('/foo', '/bar', audit=True)

    def test_invalid_deny(self):
        with self.assertRaises(AppArmorBug):
            AliasRule('/foo', '/bar', deny=True)

    def test_invalid_priority_1(self):
        with self.assertRaises(AppArmorBug):
            AliasRule('/foo', '/bar', priority=1)

    def test_invalid_priority_2(self):
        with self.assertRaises(AppArmorBug):
            AliasRule('/foo', '/bar', priority=0)


class InvalidAliasTest(AATest):
    def _check_invalid_rawrule(self, rawrule, matches_regex=False):
        obj = None
        self.assertEqual(AliasRule.match(rawrule), matches_regex)
        with self.assertRaises(AppArmorException):
            obj = AliasRule.create_instance(rawrule)

        self.assertIsNone(obj, 'AliasRule handed back an object unexpectedly')

    def test_invalid_missing_orig_path(self):
        self._check_invalid_rawrule('alias    ->  /bar ,  ', matches_regex=True)  # missing orig_path

    def test_invalid_missing_target(self):
        self._check_invalid_rawrule('alias /foo  ->   ,  ', matches_regex=True)  # missing target

    def test_invalid_net_non_AliasRule(self):
        self._check_invalid_rawrule('dbus,')  # not a alias rule


class WriteAliasTestAATest(AATest):
    tests = (
        #  raw rule                             clean rule
        ('  alias  /foo  ->  /bar,  ',          'alias /foo -> /bar,'),
        ('  alias  /foo  ->  /bar,  # comment', 'alias /foo -> /bar,'),
        ('  alias  "/foo"  ->  "/bar",  ',      'alias /foo -> /bar,'),
        ('  alias  "/foo 2"  ->  "/bar 2",  ',  'alias "/foo 2" -> "/bar 2",'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(AliasRule.match(rawrule))
        obj = AliasRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually_1(self):
        obj = AliasRule('/foo', '/bar')

        expected = '    alias /foo -> /bar,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')

    def test_write_manually_2(self):
        obj = AliasRule('/foo 2', '/bar 2')

        expected = '    alias "/foo 2" -> "/bar 2",'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class AliasCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = AliasRule.create_instance(self.rule)
        check_obj = AliasRule.create_instance(param)

        self.assertTrue(AliasRule.match(param))

        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class AliasCoveredTest_01(AliasCoveredTest):
    rule = 'alias /foo -> /bar,'

    tests = (
        #   rule                                        equal  strict equal  covered  covered exact
        ('           alias /foo -> /bar,',             (True,  True,         True,    True)),
        ('           alias   /foo   ->    /bar  ,  ',  (True,  False,        True,    True)),
        ('           alias /foo -> /bar,   # comment', (True,  False,        True,    True)),
        ('           alias /foo ->  /bar,  # comment', (True,  False,        True,    True)),
        ('           alias /foo -> /asdf,',            (False, False,        False,   False)),
        ('           alias /whatever -> /bar,',        (False, False,        False,   False)),
        ('           alias /whatever -> /asdf,',       (False, False,        False,   False)),
    )


class AliasCoveredTest_Invalid(AATest):
    # def test_borked_obj_is_covered_1(self):
    #     obj = AliasRule.create_instance('alias /foo -> /bar,')
    #
    #     testobj = AliasRule('/foo', '/bar')
    #
    #     with self.assertRaises(AppArmorBug):
    #         obj.is_covered(testobj)
    #
    # def test_borked_obj_is_covered_2(self):
    #     obj = AliasRule.create_instance('alias /foo -> /bar,')
    #
    #     testobj = AliasRule('/foo', '/bar')
    #     testobj.target = ''
    #
    #     with self.assertRaises(AppArmorBug):
    #         obj.is_covered(testobj)

    def test_invalid_is_covered_3(self):
        raw_rule = 'alias /foo -> /bar,'

        class SomeOtherClass(AliasRule):
            pass

        obj = AliasRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal(self):
        raw_rule = 'alias /foo -> /bar,'

        class SomeOtherClass(AliasRule):
            pass

        obj = AliasRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class AliasLogprofHeaderTest(AATest):
    tests = (
        ('alias /foo -> /bar,', [_('Alias'), '/foo -> /bar']),
    )

    def _run_test(self, params, expected):
        obj = AliasRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


# --- tests for AliasRuleset --- #

class AliasRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = AliasRuleset()
        ruleset_2 = AliasRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

    def test_ruleset_1(self):
        ruleset = AliasRuleset()
        rules = [
            'alias /foo -> /bar,',
            '  alias  /asdf   ->   /whatever  ,',
            'alias /asdf -> /somewhere,',
            'alias /foo -> /bar,',
        ]

        expected_raw = [
            'alias /foo -> /bar,',
            'alias  /asdf   ->   /whatever  ,',
            'alias /asdf -> /somewhere,',
            'alias /foo -> /bar,',
            '',
        ]

        expected_clean = [
            'alias /asdf -> /somewhere,',
            'alias /asdf -> /whatever,',
            'alias /foo -> /bar,',
            'alias /foo -> /bar,',
            '',
        ]

        expected_clean_unsorted = [
            'alias /foo -> /bar,',
            'alias /asdf -> /whatever,',
            'alias /asdf -> /somewhere,',
            'alias /foo -> /bar,',
            '',
        ]

        for rule in rules:
            ruleset.add(AliasRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())
        self.assertEqual(expected_clean_unsorted, ruleset.get_clean_unsorted())


class AliasGlobTestAATest(AATest):
    def setUp(self):
        self.ruleset = AliasRuleset()

#   def test_glob_1(self):
#       with self.assertRaises(NotImplementedError):
#           self.ruleset.get_glob('@{foo} = /bar')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for change_profile rules
            self.ruleset.get_glob_ext('@{foo} = /bar')


class AliasDeleteTestAATest(AATest):
    pass


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
# ------------------------------------------------------------------

import re
import unittest

import apparmor.severity as severity
from apparmor.common import AppArmorBug, hasher
from apparmor.rule import BaseRule, parse_modifiers
from common_test import AATest, setup_all_loops


class TestBaserule(AATest):

    class ValidSubclass(BaseRule):
        @classmethod
        def _create_instance(cls, raw_rule, matches):
            pass

        def get_clean(self, depth=0):
            pass

        def _is_covered_localvars(self, other_rule):
            pass

        def _is_equal_localvars(self, other_rule, strict):
            pass

        def _logprof_header_localvars(self):
            pass

    def test_implemented_abstract_methods(self):
        self.ValidSubclass()

    def test_unimplemented_abstract_methods(self):
        with self.assertRaises(TypeError):
            BaseRule()

        class InvalidSubclass(BaseRule):
            pass

        with self.assertRaises(TypeError):
            InvalidSubclass()

    def test_abstract__create_instance(self):
        with self.assertRaises(NotImplementedError):
            BaseRule._create_instance('foo', None)

    def test_abstract__create_instance_2(self):
        with self.assertRaises(AppArmorBug):
            BaseRule.create_instance('foo')

    def test_abstract__match(self):
        with self.assertRaises(AppArmorBug):
            BaseRule._match('foo')

    def test_abstract__match2(self):
        with self.assertRaises(AppArmorBug):
            BaseRule.match('foo')

    def test_abstract__match3(self):
        with self.assertRaises(NotImplementedError):
            self.ValidSubclass.match('foo')

    def test_parse_modifiers_invalid(self):
        regex = re.compile(r'^\s*(priority\s*=\s*(?P<priority>[+-]?[0-9]*)\s+)?(?P<audit>audit\s+)?(?P<allow>allow\s+|deny\s+|invalid\s+)?')
        matches = regex.search('audit invalid ')

        with self.assertRaises(AppArmorBug):
            parse_modifiers(matches)

    def test_default_severity(self):
        sev_db = severity.Severity('../severity.db', 'unknown')
        obj = self.ValidSubclass()
        rank = obj.severity(sev_db)
        self.assertEqual(rank, sev_db.NOT_IMPLEMENTED)

    def test_edit_header_localvars(self):
        obj = self.ValidSubclass()
        with self.assertRaises(NotImplementedError):
            obj.edit_header()

    def test_validate_edit_localvars(self):
        obj = self.ValidSubclass()
        with self.assertRaises(NotImplementedError):
            obj.validate_edit('/foo')

    def test_store_edit_localvars(self):
        obj = self.ValidSubclass()
        with self.assertRaises(NotImplementedError):
            obj.store_edit('/foo')

    def test_from_hashlog(self):
        obj = self.ValidSubclass()
        with self.assertRaises(NotImplementedError):
            obj.from_hashlog(hasher())

    def test_hashlog_from_event(self):
        with self.assertRaises(NotImplementedError):
            BaseRule.hashlog_from_event(None, None)


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/env python3
# ------------------------------------------------------------------
#
#  Copyright (C) 2013-2020 Canonical Ltd.
#  Authors: Steve Beattie <steve.beattie@canonical.com>
#
#  This program is free software; you can redistribute it and/or
#  modify it under the terms of version 2 of the GNU General Public
#  License published by the Free Software Foundation.
#
#  Simple test script for checking for errors and warnings emitted by
#  the apparmor parser.
#
# ------------------------------------------------------------------

import os
import subprocess
import sys
import unittest
from argparse import ArgumentParser
from shutil import rmtree
from tempfile import mkdtemp

import testlib

config = None


class AAErrorTests(testlib.AATestTemplate):
    def setUp(self):
        self.maxDiff = None
        self.cmd_prefix = [config.parser, '--config-file=./parser.conf', '-S', '-I', 'errors']

        self.tmpdir = os.path.realpath(mkdtemp(prefix='test-aa-parser-errors-'))
        self.profile_dir = os.path.join(self.tmpdir, 'profile')
        os.mkdir(self.profile_dir)


    def tearDown(self):
        if os.path.exists(self.tmpdir):
            rmtree(self.tmpdir)

    def _run_test(self, profile, message='', is_error=True):
        cmd = self.cmd_prefix + [profile]

        (rc, out, outerr) = self._run_cmd(cmd, stdout=subprocess.DEVNULL)
        report = "\nCommand: {}\nExit value:{}\nSTDERR\n{}".format(" ".join(cmd), rc, outerr)
        if is_error:
            self.assertNotEqual(rc, 0, report)
        else:
            self.assertEqual(rc, 0, report)

        ignore_messages = (
            'Cache read/write disabled: interface file missing. (Kernel needs AppArmor 2.4 compatibility patch.)\n',
        )
        for ign in ignore_messages:
            if ign in outerr:
                outerr = outerr.replace(ign, '')

        self.assertEqual(message, outerr, report)

    def test_okay(self):
        self._run_test('errors/okay.sd', is_error=False)

    def test_single(self):
        self._run_test(
            'errors/single.sd',
            "AppArmor parser error for errors/single.sd in profile errors/single.sd at line 3: Could not open 'failure'\n",
        )

    def test_double(self):
        self._run_test(
            'errors/double.sd',
            "AppArmor parser error for errors/double.sd in profile errors/includes/busted at line 66: Could not open 'does-not-exist'\n",
        )

    def test_modefail(self):
        self._run_test(
            'errors/modefail.sd',
            "AppArmor parser error for errors/modefail.sd in profile errors/modefail.sd at line 6: syntax error, unexpected TOK_ID, expecting TOK_MODE\n",
        )

    def test_multi_include(self):
        self._run_test(
            'errors/multi_include.sd',
            "AppArmor parser error for errors/multi_include.sd in profile errors/multi_include.sd at line 12: Could not open 'failure'\n",
        )

    def test_deprecation1(self):
        self.cmd_prefix.append('--warn=deprecated')
        self._run_test(
            'errors/deprecation1.sd',
            "Warning from errors/deprecation1.sd (errors/deprecation1.sd line 6): The use of file paths as profile names is deprecated. See man apparmor.d for more information\n",
            is_error=False
        )

    def test_deprecation2(self):
        self.cmd_prefix.append('--warn=deprecated')
        self._run_test(
            'errors/deprecation2.sd',
            "Warning from errors/deprecation2.sd (errors/deprecation2.sd line 6): The use of file paths as profile names is deprecated. See man apparmor.d for more information\n",
            is_error=False
        )

    def test_non_existant_profile(self):
        test_profile = os.path.join(self.profile_dir, "does-not-exist.sd")
        self._run_test(
            test_profile,
            "File {} not found, skipping...\n".format(test_profile),
        )

    # We can run this test with multiple different arguments
    def _test_non_existant_symlink_target(self):
        """Helper Function to test the parser on a symlink with a non-existent target"""

        test_profile = os.path.join(self.profile_dir, "non-existant-target.sd")
        os.symlink('does-not-exist.sd', test_profile)
        self._run_test(
            test_profile,
            "File {} not found, skipping...\n".format(test_profile),
        )

    def test_non_existant_symlink_target(self):
        '''Basic symlink test that goes nowhere'''
        self._test_non_existant_symlink_target()

    def test_non_existant_symlink_target_j0(self):
        '''Basic symlink test that goes nowhere with 0 jobs'''
        self.cmd_prefix.append('-j0')
        self._test_non_existant_symlink_target()

    def test_non_existant_symlink_target_j1(self):
        '''Basic symlink test that goes nowhere with 1 job arg'''
        self.cmd_prefix.append('-j1')
        self._test_non_existant_symlink_target()

    def test_non_existant_symlink_target_j8(self):
        '''Basic symlink test that goes nowhere with 8 job arg'''
        self.cmd_prefix.append('-j8')
        self._test_non_existant_symlink_target()

    def test_non_existant_symlink_target_jauto(self):
        '''Basic symlink test that goes nowhere with auto job arg'''
        self.cmd_prefix.append('-jauto')
        self._test_non_existant_symlink_target()

    def test_non_existant_symlink_target_in_directory(self):
        '''Symlink test passing a directory to the parser'''
        test_profile = os.path.join(self.profile_dir, "non-existant-target.sd")
        os.symlink('does-not-exist.sd', test_profile)
        self._run_test(
            self.profile_dir,
            "There was an error while loading profiles from {}\n".format(self.profile_dir),
        )

def main():
    global config
    p = ArgumentParser()
    p.add_argument('-p', '--parser', default=testlib.DEFAULT_PARSER, action="store", dest='parser',
                   help="Specify path of apparmor parser to use [default = %(default)s]")
    config, args = p.parse_known_args()

    unittest.main(argv=sys.argv[:1] + args)

if __name__ == "__main__":
    main()



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2020 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.common import AppArmorBug, AppArmorException
from apparmor.rule.boolean import BooleanRule, BooleanRuleset
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops

_ = init_translation()

exp = namedtuple('exp', ('comment', 'varname', 'value'))

# --- tests for single BooleanRule --- #


class BooleanTest(AATest):
    def _compare_obj(self, obj, expected):
        # boolean variables don't support the allow, audit, deny, or priority keyword
        self.assertEqual(False, obj.allow_keyword)
        self.assertEqual(False, obj.audit)
        self.assertEqual(False, obj.deny)
        self.assertEqual(None, obj.priority)

        self.assertEqual(expected.varname, obj.varname)
        self.assertEqual(expected.value, obj.value)
        self.assertEqual(expected.comment, obj.comment)


class BooleanTestParse(BooleanTest):
    tests = (
        # rawrule                                   comment    varname  value
        ('$foo=true',                        exp('',           '$foo',  'true')),
        ('$foo = false',                     exp('',           '$foo',  'false')),
        ('$foo=TrUe',                        exp('',           '$foo',  'true')),
        ('$foo = FaLsE',                     exp('',           '$foo',  'false')),
        ('  $foo =   true   ',               exp('',           '$foo',  'true')),
        ('  $foo =   true        # comment', exp(' # comment', '$foo',  'true')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(BooleanRule.match(rawrule))
        obj = BooleanRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class BooleanTestParseInvalid(BooleanTest):
    tests = (
        # rawrule                     matches regex  exception
        ('$foo =',                   (False,         AppArmorException)),
        ('$ foo =      # comment',   (False,         AppArmorException)),
        ('${foo =      ',            (False,         AppArmorException)),
        # XXX RE_PROFILE_BOOLEAN allows a trailing comma even if the parser disallows it
        # ('$foo = true,',             (True,          AppArmorException)),  # trailing comma
        # ('$foo = false   ,  ',       (True,          AppArmorException)),  # trailing comma
        # ('$foo = true,   # comment', (True,          AppArmorException)),  # trailing comma
    )

    def _run_test(self, rawrule, expected):
        self.assertEqual(BooleanRule.match(rawrule), expected[0])
        with self.assertRaises(expected[1]):
            BooleanRule.create_instance(rawrule)


class BooleanFromInit(BooleanTest):
    # tests = (
    #     # BooleanRule object                              comment  varname  value
    #     (BooleanRule('$foo', True),                   exp('',      '$foo',  True)),
    #     (BooleanRule('$foo', False),                  exp('',      '$foo',  False)),
    #     (BooleanRule('$foo', True,  comment='# cmt'), exp('# cmt', '$foo',  True)),
    #     (BooleanRule('$foo', False, comment='# cmt'), exp('# cmt', '$foo',  False)),
    # )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidBooleanInit(AATest):
    tests = (
        # init params         expected exception
        ((None,     True),    AppArmorBug),        # varname not a str
        (('',       True),    AppArmorException),  # empty varname
        (('foo',    True),    AppArmorException),  # varname not starting with '$'
        (('foo',    True),    AppArmorException),  # varname not starting with '$'

        (('$foo',   None),    AppArmorBug),        # value not a string
        (('$foo',   ''),      AppArmorException),  # empty value
        (('$foo',   'maybe'), AppArmorException),  # invalid value
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            BooleanRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            BooleanRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            BooleanRule('$foo')

    def test_invalid_audit(self):
        with self.assertRaises(AppArmorBug):
            BooleanRule('$foo', 'true', audit=True)

    def test_invalid_deny(self):
        with self.assertRaises(AppArmorBug):
            BooleanRule('$foo', 'true', deny=True)

    def test_invalid_priority_1(self):
        with self.assertRaises(AppArmorBug):
            BooleanRule('$foo', 'true', priority=-100)

    def test_invalid_priority_2(self):
        with self.assertRaises(AppArmorBug):
            BooleanRule('$foo', 'true', priority=0)


class InvalidBooleanTest(AATest):
    def _check_invalid_rawrule(self, rawrule, matches_regex=False):
        obj = None
        self.assertEqual(BooleanRule.match(rawrule), matches_regex)
        with self.assertRaises(AppArmorException):
            obj = BooleanRule.create_instance(rawrule)

        self.assertIsNone(obj, 'BooleanRule handed back an object unexpectedly')

    def test_invalid_missing_value(self):
        self._check_invalid_rawrule('$foo =  ', matches_regex=False)  # missing value

    def test_invalid_net_non_BooleanRule(self):
        self._check_invalid_rawrule('dbus,')  # not a boolean rule


class WriteBooleanTestAATest(AATest):
    tests = (
        #  raw rule                            clean rule
        ('  $foo  =  true   ',                 '$foo = true'),
        ('  $foo  =  true   # comment',        '$foo = true'),
        ('  $foo   =    false   ',             '$foo = false'),
        ('  $foo   =    false      # comment', '$foo = false'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(BooleanRule.match(rawrule))
        obj = BooleanRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually_1(self):
        obj = BooleanRule('$foo', 'true')

        expected = '    $foo = true'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')

    def test_write_manually_2(self):
        obj = BooleanRule('$foo', 'false')

        expected = '    $foo = false'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class BooleanCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = BooleanRule.create_instance(self.rule)
        check_obj = BooleanRule.create_instance(param)

        self.assertTrue(BooleanRule.match(param))

        self.assertEqual(obj.is_equal(check_obj), expected[0], 'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(obj.is_equal(check_obj, True), expected[1], 'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(obj.is_covered(check_obj), expected[2], 'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(obj.is_covered(check_obj, True, True), expected[3], 'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class BooleanCoveredTest_01(BooleanCoveredTest):
    rule = '$foo = true'

    tests = (
        #   rule                                equal  strict equal  covered  covered exact
        ('           $foo = true',             (True,  True,         True,    True)),
        ('           $foo = TRUE',             (True,  False,        True,    True)),  # upper vs. lower case
        ('           $foo = true   # comment', (True,  False,        True,    True)),
        ('           $foo = false',            (False, False,        False,   False)),
        ('           $foo = false     # cmt',  (False, False,        False,   False)),
        ('           $bar = true',             (False, False,        False,   False)),  # different variable name
    )


class BooleanCoveredTest_02(BooleanCoveredTest):
    rule = '$foo = false'

    tests = (
        #   rule                                equal  strict equal  covered  covered exact
        ('           $foo = false',            (True,  True,         True,    True)),
        ('           $foo = false  # comment', (True,  False,        True,    True)),
        ('           $foo = true',             (False, False,        False,   False)),
        ('           $foo = true      # cmt',  (False, False,        False,   False)),
        ('           $bar = false',            (False, False,        False,   False)),  # different variable name
    )


class BooleanCoveredTest_Invalid(AATest):
    def test_borked_obj_is_covered_2(self):
        obj = BooleanRule.create_instance('$foo = true')

        testobj = BooleanRule('$foo', 'true')
        testobj.value = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_covered_3(self):
        raw_rule = '$foo = true'

        class SomeOtherClass(BooleanRule):
            pass

        obj = BooleanRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal(self):
        raw_rule = '$foo = true'

        class SomeOtherClass(BooleanRule):
            pass

        obj = BooleanRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class BooleanLogprofHeaderTest(AATest):
    tests = (
        ('$foo = true', [_('Boolean Variable'), '$foo = true']),
    )

    def _run_test(self, params, expected):
        obj = BooleanRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


# --- tests for BooleanRuleset --- #

class BooleanRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = BooleanRuleset()
        ruleset_2 = BooleanRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

    def test_ruleset_1(self):
        ruleset = BooleanRuleset()
        rules = [
            '$foo = true',
            '$baz= false',
        ]

        expected_raw = [
            '$foo = true',
            '$baz= false',
            '',
        ]

        expected_clean = [
            '$baz = false',
            '$foo = true',
            '',
        ]

        expected_clean_unsorted = [
            '$foo = true',
            '$baz = false',
            '',
        ]

        for rule in rules:
            ruleset.add(BooleanRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())
        self.assertEqual(expected_clean_unsorted, ruleset.get_clean_unsorted())

    def test_ruleset_overwrite(self):
        ruleset = BooleanRuleset()

        ruleset.add(BooleanRule.create_instance('$foo = true'))
        with self.assertRaises(AppArmorException):
            ruleset.add(BooleanRule.create_instance('$foo = false'))  # attempt to redefine @{foo}


class BooleanGlobTestAATest(AATest):
    def setUp(self):
        self.ruleset = BooleanRuleset()

#   def test_glob_1(self):
#       with self.assertRaises(NotImplementedError):
#           self.ruleset.get_glob('$foo = true')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for boolean rules
            self.ruleset.get_glob_ext('$foo = true')


class BooleanDeleteTestAATest(AATest):
    pass


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2022 Canonical, Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.logparser import ReadLog

from common_test import AATest, setup_all_loops

from apparmor.rule.userns import UserNamespaceRule, UserNamespaceRuleset
from apparmor.common import AppArmorException, AppArmorBug, hasher
from apparmor.translations import init_translation
_ = init_translation()


class UserNamespaceTestParse(AATest):
    tests = (
        #                                          access                 audit  deny   allow  comment
        ('userns,',              UserNamespaceRule(UserNamespaceRule.ALL, False, False, False, '')),
        ('userns create,',       UserNamespaceRule(('create'),            False, False, False, '')),
        ('audit userns create,', UserNamespaceRule(('create'),            True,  False, False, '')),
        ('deny userns,',         UserNamespaceRule(UserNamespaceRule.ALL, False, True,  False, '')),
        ('audit allow userns,',  UserNamespaceRule(UserNamespaceRule.ALL, True,  False, True,  '')),
        ('userns create, # cmt', UserNamespaceRule(('create'),            False, False, False, ' # cmt')),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(UserNamespaceRule.match(rawrule))
        obj = UserNamespaceRule.create_instance(rawrule)
        expected.raw_rule = rawrule.strip()
        self.assertTrue(obj.is_equal(expected, True))


class UserNamespaceTestParseInvalid(AATest):
    tests = (
        ('userns invalidaccess,', AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(UserNamespaceRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            UserNamespaceRule.create_instance(rawrule)

    def test_parse_fail(self):
        with self.assertRaises(AppArmorException):
            UserNamespaceRule.create_instance('foo,')

    def test_diff_non_usernsrule(self):
        exp = namedtuple('exp', ('audit', 'deny', 'priority'))
        obj = UserNamespaceRule(('create'))
        with self.assertRaises(AppArmorBug):
            obj.is_equal(exp(False, False, None), False)

    def test_diff_access(self):
        obj1 = UserNamespaceRule(UserNamespaceRule.ALL)
        obj2 = UserNamespaceRule(('create'))
        self.assertFalse(obj1.is_equal(obj2, False))


class InvalidUserNamespaceInit(AATest):
    tests = (
        # init params  expected exception
        ((''),         TypeError),          # empty access
        (('    '),     AppArmorBug),        # whitespace access
        (('xyxy'),     AppArmorException),  # invalid access
        (dict(),       TypeError),          # wrong type for access
        (None,         TypeError),          # wrong type for access
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            UserNamespaceRule(*params)

    def test_missing_params(self):
        with self.assertRaises(TypeError):
            UserNamespaceRule()


class WriteUserNamespaceTestAATest(AATest):
    tests = (
        #  raw rule                              clean rule
        ('     userns         ,    # foo    ',   'userns, # foo'),
        ('    audit     userns create,',         'audit userns create,'),
        ('   deny userns      ,# foo bar',       'deny userns, # foo bar'),
        ('   allow userns  create   ,# foo bar', 'allow userns create, # foo bar'),
        ('userns,',                              'userns,'),
        ('userns create,',                       'userns create,'),
        (' priority = -1 allow userns  create,', 'priority=-1 allow userns create,'),
        (' priority =  0 allow userns  create,', 'priority=0 allow userns create,'),
        (' priority=+234 allow userns  create,', 'priority=234 allow userns create,'),
        (' priority = 65 allow userns  create,', 'priority=65 allow userns create,'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(UserNamespaceRule.match(rawrule))
        obj = UserNamespaceRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually(self):
        obj = UserNamespaceRule('create', allow_keyword=True)

        expected = '    allow userns create,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')

    def test_write_invalid_access(self):
        obj = UserNamespaceRule('create')
        obj.access = ''
        with self.assertRaises(AppArmorBug):
            obj.get_clean()


class UserNamespaceIsCoveredTest(AATest):
    def test_is_covered(self):
        obj = UserNamespaceRule(UserNamespaceRule.ALL)
        self.assertTrue(obj.is_covered(UserNamespaceRule(('create'))))
        self.assertTrue(obj.is_covered(UserNamespaceRule(UserNamespaceRule.ALL)))

    def test_is_not_covered(self):
        obj = UserNamespaceRule(('create'))
        self.assertFalse(obj.is_covered(UserNamespaceRule(UserNamespaceRule.ALL)))


class UserNamespaceLogprofHeaderTest(AATest):
    tests = (
        ('userns,',        [_('Access mode'), _('ALL')]),
        ('userns create,', [_('Access mode'), 'create']),
    )

    def _run_test(self, params, expected):
        obj = UserNamespaceRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)

    def test_unconfined_usens_from_log(self):

        log = 'type=AVC msg=audit(1720613712.153:168): apparmor="AUDIT" operation="userns_create" class="namespace" info="Userns create - transitioning profile" profile="unconfined" pid=5630 comm="unshare" requested="userns_create" target="unprivileged_userns" execpath="/usr/bin/unshare"'
        parser = ReadLog('', '', '')

        hl = hasher()

        ev = parser.parse_event(log)
        UserNamespaceRule.hashlog_from_event(hl, ev)

        expected = {'create': True}
        self.assertEqual(hl, expected)

        ur = UserNamespaceRule.from_hashlog(hl)

        expected = UserNamespaceRule('create')

        self.assertTrue(expected.is_equal(next(ur)))
        with self.assertRaises(StopIteration):
            next(ur)


class UserNamespaceGlobTestAATest(AATest):
    def test_glob(self):
        self.assertEqual(UserNamespaceRuleset().get_glob('userns create,'), 'userns,')


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#! /usr/bin/python3
# ------------------------------------------------------------------
#
#    Copyright (C) 2025 Canonical Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest

from apparmor.common import AppArmorException
from apparmor.rule.capability import CapabilityRule
from apparmor.rule.change_profile import ChangeProfileRule
from apparmor.rule.dbus import DbusRule
from apparmor.rule.file import FileRule
from apparmor.rule.io_uring import IOUringRule
from apparmor.rule.mount import MountRule
from apparmor.rule.mqueue import MessageQueueRule
from apparmor.rule.network import NetworkRule
from apparmor.rule.pivot_root import PivotRootRule
from apparmor.rule.ptrace import PtraceRule
from apparmor.rule.signal import SignalRule
from apparmor.rule.unix import UnixRule
from apparmor.rule.userns import UserNamespaceRule
from apparmor.rule.all import AllRule
from common_test import AATest, setup_all_loops


class TestInvalid_parse_priority(AATest):
    tests = (
        ((CapabilityRule, 'priority=a capability,'), AppArmorException),
        ((DbusRule, 'priority=a dbus,'), AppArmorException),
        ((MountRule, 'priority=a mount,'), AppArmorException),
        ((MountRule, 'priority=a umount,'), AppArmorException),
        ((MountRule, 'priority=a unmount,'), AppArmorException),
        ((MountRule, 'priority=a remount,'), AppArmorException),
        ((SignalRule, 'priority=a signal,'), AppArmorException),
        ((PtraceRule, 'priority=a ptrace,'), AppArmorException),
        ((PivotRootRule, 'priority=a pivot_root,'), AppArmorException),
        ((UnixRule, 'priority=a unix,'), AppArmorException),
        ((NetworkRule, 'priority=a network,'), AppArmorException),
        ((UserNamespaceRule, 'priority=a userns,'), AppArmorException),
        ((MessageQueueRule, 'priority=a mqueue,'), AppArmorException),
        ((IOUringRule, 'priority=a io_uring,'), AppArmorException),
        ((ChangeProfileRule, 'priority=a change_profile,'), AppArmorException),
        ((FileRule, 'priority=a file,'), AppArmorException),
        ((AllRule, 'priority=a all,'), AppArmorException),
    )

    def _run_test(self, params, expected):
        rule_cls, rule = params
        with self.assertRaises(expected):
            rule_cls.create_instance(rule)  # Invalid rule


class TestInvalid_init_priority(AATest):
    tests = (
        ((CapabilityRule, (CapabilityRule.ALL,)), AppArmorException),
        ((DbusRule, (DbusRule.ALL,) * 8), AppArmorException),
        ((MountRule, (MountRule.ALL,) * 5), AppArmorException),
        ((SignalRule, (SignalRule.ALL,) * 3), AppArmorException),
        ((PtraceRule, (PtraceRule.ALL,) * 2), AppArmorException),
        ((PivotRootRule, (PivotRootRule.ALL,) * 3), AppArmorException),
        ((UnixRule, (UnixRule.ALL,) * 4), AppArmorException),
        ((NetworkRule, (NetworkRule.ALL,) * 5), AppArmorException),
        ((UserNamespaceRule, (UserNamespaceRule.ALL,) * 1), AppArmorException),
        ((MessageQueueRule, (MessageQueueRule.ALL,) * 4), AppArmorException),
        ((IOUringRule, (IOUringRule.ALL,) * 2), AppArmorException),
        ((ChangeProfileRule, (ChangeProfileRule.ALL,) * 3), AppArmorException),
        ((FileRule, (FileRule.ALL,) * 5), AppArmorException),
        ((AllRule, ()), AppArmorException),
    )

    def _run_test(self, params, expected):
        rule_cls, args = params
        with self.assertRaises(expected):
            rule_cls(*args, priority="invalid")  # ValueError


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.common import AppArmorBug, AppArmorException, cmd
from apparmor.logparser import ReadLog
from apparmor.rule.network import NetworkRule, NetworkRuleset, network_domain_keywords, network_ipv6
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops
import re

_ = init_translation()

exp = namedtuple('exp', ('audit', 'allow_keyword', 'deny', 'comment',
                         'accesses', 'domain', 'all_domains', 'type_or_protocol',
                         'all_type_or_protocols', 'local_expr', 'peer_expr'))

# --- check if the keyword list is up to date --- #


class NetworkKeywordsTest(AATest):
    def test_network_keyword_list(self):
        rc, output = cmd('../../common/list_af_names.sh')
        self.assertEqual(rc, 0)

        af_names = []
        af_pairs = output.replace('AF_', '').strip().lower().split(",")
        for af_pair in af_pairs:
            af_name = af_pair.lstrip().split(" ")[0]
            # skip max af name definition
            if af_name and af_name != "max":
                af_names.append(af_name)

        missing_af_names = []
        for keyword in af_names:
            if keyword not in network_domain_keywords:
                # keywords missing in the system are ok (= older kernel), but network_domain_keywords needs to have the full list
                missing_af_names.append(keyword)

        self.assertEqual(
            missing_af_names, [],
            'Missing af_names in NetworkRule network_domain_keywords. This test is likely running '
            'on an newer kernel and will require updating the list of network domain keywords in '
            'utils/apparmor/rule/network.py')


class NetworkPV6Test(AATest):
    def test_ipv6(self):
        tests = [
            ("2001:0db8:85a3:0000:0000:8a2e:0370:7334", True),  # Standard IPv6
            ("2001:db8::8a2e:370:7334", True),  # Zero Compression
            ("::1", True),  # IPv6 Loopback
            ("::", True),  # IPv6 Unspecified
            ("::ffff:192.168.236.159", True),  # IPv6-mapped IPv4
            ("fe80::1ff:fe23:4567:890a%eth0", True),  # IPv6 with Zone Identifier
            ("1234:5678::abcd:ef12:3456", True),  # Mixed groups and zero compression
            ("12345::6789", False),  # Erroneous IP (invalid hex group length)
            ("192.168.1.1", False),  # IPv4 only
        ]

        for test in tests:
            self.assertEqual(bool(re.match(network_ipv6, test[0])), test[1])


# --- tests for single NetworkRule --- #
class NetworkTest(AATest):
    def _compare_obj(self, obj, expected):
        self.assertEqual(expected.allow_keyword, obj.allow_keyword)
        self.assertEqual(expected.audit, obj.audit)
        self.assertEqual(expected.accesses, obj.accesses)
        self.assertEqual(expected.domain, obj.domain)
        self.assertEqual(expected.type_or_protocol, obj.type_or_protocol)
        self.assertEqual(expected.all_domains, obj.all_domains)
        self.assertEqual(expected.all_type_or_protocols, obj.all_type_or_protocols)
        self.assertEqual(expected.deny, obj.deny)
        self.assertEqual(expected.comment, obj.comment)
        self.assertEqual(expected.local_expr, obj.local_expr)
        self.assertEqual(expected.peer_expr, obj.peer_expr)


class NetworkTestParse(NetworkTest):
    tests = (
        # rawrule                                       audit  allow  deny   comment       access               domain  all?   type/proto  all?   local_expr                             peer_expr
        ('network,',                                exp(False, False, False, '',           None,                None,   True,  None,       True,  NetworkRule.ALL,                       NetworkRule.ALL)),
        ('network inet,',                           exp(False, False, False, '',           None,                'inet', False, None,       True,  NetworkRule.ALL,                       NetworkRule.ALL)),
        ('network inet stream,',                    exp(False, False, False, '',           None,                'inet', False, 'stream',   False, NetworkRule.ALL,                       NetworkRule.ALL)),
        ('deny network inet stream, # comment',     exp(False, False, True,  ' # comment', None,                'inet', False, 'stream',   False, NetworkRule.ALL,                       NetworkRule.ALL)),
        ('audit allow network tcp,',                exp(True,  True,  False, '',           None,                None,   True,  'tcp',      False, NetworkRule.ALL,                       NetworkRule.ALL)),
        ('network stream,',                         exp(False, False, False, '',           None,                None,   True,  'stream',   False, NetworkRule.ALL,                       NetworkRule.ALL)),
        ('network stream peer=(ip=::1 port=22),',   exp(False, False, False, '',           None,                None,   True,  'stream',   False, NetworkRule.ALL,                       {"ip": "::1", 'port': '22'},)),
        ('network stream ip=::1 port=22,',          exp(False, False, False, '',           None,                None,   True,  'stream',   False, {"ip": "::1", 'port': '22'},           NetworkRule.ALL)),
        ('network (bind,listen) stream,',           exp(False, False, False, '',           {'listen', 'bind'},  None,   True,  'stream',   False, NetworkRule.ALL,                       NetworkRule.ALL)),
        ('network (connect, rw) stream ip=192.168.122.2 port=22 peer=(ip=192.168.122.3 port=22),',
                                                    exp(False, False, False, '',           {'connect', 'rw'},   None,   True,  'stream',   False, {'ip': '192.168.122.2', 'port': '22'}, {"ip": "192.168.122.3", 'port': '22'})),  # noqa: E127
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(NetworkRule.match(rawrule))
        obj = NetworkRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class NetworkTestParseInvalid(NetworkTest):
    tests = (
        ('network foo,',                                    AppArmorException),
        ('network foo bar,',                                AppArmorException),
        ('network foo tcp,',                                AppArmorException),
        ('network inet bar,',                               AppArmorException),
        ('network ip=999.999.999.999,',                     AppArmorException),
        ('network port=99999,',                             AppArmorException),
        ('network inet ip=in:va::li:d0,',                   AppArmorException),
        ('network inet ip=in:va::li:d0,',                   AppArmorException),
        ('network inet ip=1:2:3:4:5:6:7:8:9:0:0:0,',        AppArmorException),  # too many segments
        ('network inet peer=(ip=1:2:3:4:5:6:7:8:9:0:0:0),', AppArmorException),  # too many segments
        ('network packet ip=1::,',                          AppArmorException),  # Only inet[6] domains can be used in conjunction with a local expression
        ('network packet peer=(ip=1::),',                   AppArmorException),  # Only inet[6] domains can be used in conjunction with a peer expression
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(NetworkRule.match(rawrule))  # the above invalid rules still match the main regex!
        with self.assertRaises(expected):
            NetworkRule.create_instance(rawrule)


class NetworkTestParseFromLog(NetworkTest):
    def test_net_from_log(self):
        parser = ReadLog('', '', '')
        event = 'type=AVC msg=audit(1428699242.551:386): apparmor="DENIED" operation="create" profile="/bin/ping" pid=10589 comm="ping" family="inet" sock_type="raw" protocol=1 lport=1234'

        parsed_event = parser.parse_event(event)

        self.assertEqual(parsed_event, {
            'request_mask': None,
            'denied_mask': None,
            'error_code': 0,
            'family': 'inet',
            'magic_token': 0,
            'parent': 0,
            'profile': '/bin/ping',
            'protocol': 'icmp',
            'sock_type': 'raw',
            'operation': 'create',
            'resource': None,
            'info': None,
            'aamode': 'REJECTING',
            'accesses': None,
            'addr': None,
            'peer_addr': None,
            'port': 1234,
            'remote_port': None,
            'time': 1428699242,
            'active_hat': None,
            'pid': 10589,
            'task': 0,
            'attr': None,
            'name2': None,
            'name': None,
            'class': None,
        })

        obj = NetworkRule(NetworkRule.ALL, parsed_event['family'], parsed_event['sock_type'], NetworkRule.ALL, NetworkRule.ALL, log_event=parsed_event)

        #              audit  allow  deny   comment  domain  all?   type/proto  all?
        expected = exp(False, False, False, '',     None, 'inet', False, 'raw',      False, NetworkRule.ALL, NetworkRule.ALL)

        self._compare_obj(obj, expected)

        self.assertEqual(obj.get_raw(1), '  network inet raw,')


class NetworkFromInit(NetworkTest):
    tests = (
        # NetworkRule object                                                                                                   audit  allow  deny   comment access               domain  all?   type/proto  all?     Local expr          Peer expr
        (NetworkRule(NetworkRule.ALL,    'inet',           'raw',           NetworkRule.ALL, NetworkRule.ALL, deny=True),  exp(False, False, True,  '',      None,               'inet', False, 'raw',     False, NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule(NetworkRule.ALL,    'inet',           'raw',           NetworkRule.ALL, NetworkRule.ALL),             exp(False, False, False, '',      None,               'inet', False, 'raw',     False, NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule(NetworkRule.ALL,    'inet',           NetworkRule.ALL, NetworkRule.ALL, NetworkRule.ALL),             exp(False, False, False, '',      None,               'inet', False, None,      True,  NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule(NetworkRule.ALL,    NetworkRule.ALL,  NetworkRule.ALL, NetworkRule.ALL, NetworkRule.ALL),             exp(False, False, False, '',      None,               None,   True,  None,      True,  NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule(NetworkRule.ALL,    NetworkRule.ALL, 'tcp',            NetworkRule.ALL, NetworkRule.ALL),             exp(False, False, False, '',      None,               None,   True,  'tcp',     False, NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule(NetworkRule.ALL,    NetworkRule.ALL, 'stream',         NetworkRule.ALL, NetworkRule.ALL),             exp(False, False, False, '',      None,               None,   True,  'stream',  False, NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule('bind',             NetworkRule.ALL, 'stream',         NetworkRule.ALL, NetworkRule.ALL),             exp(False, False, False, '',      {'bind'},           None,   True,  'stream',  False, NetworkRule.ALL,  NetworkRule.ALL)),
        (NetworkRule({'bind', 'listen'}, NetworkRule.ALL, 'stream',         {'port': '22'},  NetworkRule.ALL),             exp(False, False, False, '',      {'bind', 'listen'}, None,   True,  'stream',  False, {'port': '22'},  NetworkRule.ALL)),
        (NetworkRule(NetworkRule.ALL,    NetworkRule.ALL, 'stream',         NetworkRule.ALL, {'port': '22'}),              exp(False, False, False, '',      None,               None,   True,  'stream',  False, NetworkRule.ALL,  {'port': '22'})),
        (NetworkRule(NetworkRule.ALL,    NetworkRule.ALL, 'stream',         NetworkRule.ALL, {'ip': '::1', 'port': '22'}), exp(False, False, False, '',      None,               None,   True,  'stream',  False, NetworkRule.ALL,  {'ip': '::1', 'port': '22'})),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidNetworkInit(AATest):
    tests = (
        # init params                                                           expected exception
        ((NetworkRule.ALL,   'inet', '',     NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # empty type_or_protocol
        ((NetworkRule.ALL,   '',     'tcp',  NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # empty domain
        ((NetworkRule.ALL,   '    ', 'tcp',  NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # whitespace domain
        ((NetworkRule.ALL,   'inet', '   ',  NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # whitespace type_or_protocol
        ((NetworkRule.ALL,   'xyxy', 'tcp',  NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # invalid domain
        ((NetworkRule.ALL,   'inet', 'xyxy', NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # invalid type_or_protocol
        ((NetworkRule.ALL,   dict(), 'tcp',  NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # wrong type for domain
        ((NetworkRule.ALL,   None,   'tcp',  NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # wrong type for domain
        ((NetworkRule.ALL,   'inet', dict(), NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # wrong type for type_or_protocol
        ((NetworkRule.ALL,   'inet', None,   NetworkRule.ALL, NetworkRule.ALL),  AppArmorBug),        # wrong type for type_or_protocol
        (('invalid_access',  'inet', None,   NetworkRule.ALL, NetworkRule.ALL),  AppArmorException),  # Invalid Access
        (({'bind', 'invld'}, 'inet', None,   NetworkRule.ALL, NetworkRule.ALL),  AppArmorException),  # Invalid Access
        ((NetworkRule.ALL,   'inet', None,   {'ip': ':::::'}, NetworkRule.ALL),  AppArmorException),  # Invalid ip in local expression
        ((NetworkRule.ALL,   'inet', None,   NetworkRule.ALL, {'ip': ':::::'}),  AppArmorException),  # Invalid ip in peer expression
        ((NetworkRule.ALL,   'inet', None,   {'invld': '0'},  NetworkRule.ALL),  AppArmorException),  # Invalid keyword in local expression
        ((NetworkRule.ALL,   'inet', None,   NetworkRule.ALL, {'invld': '0'}),   AppArmorException),  # Invalid keyword in peer expression
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            NetworkRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            NetworkRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            NetworkRule('inet')


class InvalidNetworkTest(AATest):
    def _check_invalid_rawrule(self, rawrule):
        obj = None
        self.assertFalse(NetworkRule.match(rawrule))
        with self.assertRaises(AppArmorException):
            obj = NetworkRule.create_instance(rawrule)

        self.assertIsNone(obj, 'NetworkRule handed back an object unexpectedly')

    def test_invalid_net_missing_comma(self):
        self._check_invalid_rawrule('network')  # missing comma

    def test_invalid_net_non_NetworkRule(self):
        self._check_invalid_rawrule('dbus,')  # not a network rule

    def test_empty_net_data_1(self):
        obj = NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL)
        obj.domain = ''
        # no domain set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)

    def test_empty_net_data_2(self):
        obj = NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL)
        obj.type_or_protocol = ''
        # no type_or_protocol set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)


class WriteNetworkTestAATest(AATest):
    def _run_test(self, rawrule, expected):
        self.assertTrue(NetworkRule.match(rawrule))
        obj = NetworkRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    tests = (
        #  raw rule                                                          clean rule
        ('     network         ,    # foo        ',                          'network, # foo'),
        ('    audit     network inet,',                                      'audit network inet,'),
        ('   deny network         inet      stream,# foo bar',               'deny network inet stream, # foo bar'),
        ('   deny network         inet      ,# foo bar',                     'deny network inet, # foo bar'),
        ('   allow network         tcp      ,# foo bar',                     'allow network tcp, # foo bar'),
        ('   network     stream    peer  =  (  ip=::1  port=22  )  ,',       'network stream peer=(ip=::1 port=22),'),
        ('   network   (  bind , listen  ) stream  ip  =  ::1 port  = 22 ,', 'network (bind, listen) stream ip=::1 port=22,'),
        ('   allow network         tcp      ,# foo bar',                     'allow network tcp, # foo bar'),
        (' priority =  -02  allow network         tcp      ,# foo bar',       'priority=-2 allow network tcp, # foo bar'),
        (' priority = 0     allow network         tcp      ,# foo bar',       'priority=0 allow network tcp, # foo bar'),
        (' priority = 43    allow network         tcp      ,# foo bar',       'priority=43 allow network tcp, # foo bar'),
        (' priority=+123    allow network         tcp      ,# foo bar',       'priority=123 allow network tcp, # foo bar'),

    )

    def test_write_manually(self):
        obj = NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL,  allow_keyword=True)

        expected = '    allow network inet stream,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class NetworkCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = NetworkRule.create_instance(self.rule)
        check_obj = NetworkRule.create_instance(param)

        self.assertTrue(NetworkRule.match(param))

        self.assertEqual(
            obj.is_equal(check_obj), expected[0],
            'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(
            obj.is_equal(check_obj, True), expected[1],
            'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(
            obj.is_covered(check_obj), expected[2],
            'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(
            obj.is_covered(check_obj, True, True), expected[3],
            'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class NetworkCoveredTest_01(NetworkCoveredTest):
    rule = 'network inet,'

    tests = (
        #   rule                      equal  strict equal  covered  covered exact
        ('network,',                 (False, False,        False,   False)),
        ('network inet,',            (True,  True,         True,    True)),
        ('network inet, # comment',  (True,  False,        True,    True)),
        ('allow network inet,',      (True,  False,        True,    True)),
        ('network     inet,',        (True,  False,        True,    True)),
        ('network inet stream,',     (False, False,        True,    True)),
        ('network inet tcp,',        (False, False,        True,    True)),
        ('audit network inet,',      (False, False,        False,   False)),
        ('audit network,',           (False, False,        False,   False)),
        ('network unix,',            (False, False,        False,   False)),
        ('network tcp,',             (False, False,        False,   False)),
        ('audit deny network inet,', (False, False,        False,   False)),
        ('deny network inet,',       (False, False,        False,   False)),
    )


class NetworkCoveredTest_02(NetworkCoveredTest):
    rule = 'audit network inet,'

    tests = (
        #   rule                        equal  strict equal  covered  covered exact
        ('      network inet,',        (False, False,        True,    False)),
        ('audit network inet,',        (True,  True,         True,    True)),
        ('      network inet stream,', (False, False,        True,    False)),
        ('audit network inet stream,', (False, False,        True,    True)),
        ('      network,',             (False, False,        False,   False)),
        ('audit network,',             (False, False,        False,   False)),
        ('      network unix,',        (False, False,        False,   False)),
    )


class NetworkCoveredTest_03(NetworkCoveredTest):
    rule = 'network inet stream,'

    tests = (
        #   rule                        equal  strict equal  covered  covered exact
        ('      network inet stream,', (True,  True,         True,    True)),
        ('allow network inet stream,', (True,  False,        True,    True)),
        ('      network inet,',        (False, False,        False,   False)),
        ('      network,',             (False, False,        False,   False)),
        ('      network inet tcp,',    (False, False,        False,   False)),
        ('audit network,',             (False, False,        False,   False)),
        ('audit network inet stream,', (False, False,        False,   False)),
        ('      network unix,',        (False, False,        False,   False)),
        ('      network,',             (False, False,        False,   False)),
    )


class NetworkCoveredTest_04(NetworkCoveredTest):
    rule = 'network,'

    tests = (
        #   rule                         equal  strict equal  covered  covered exact
        ('      network,',              (True,  True,         True,    True)),
        ('allow network,',              (True,  False,        True,    True)),
        ('      network inet,',         (False, False,        True,    True)),
        ('      network inet6 stream,', (False, False,        True,    True)),
        ('      network tcp,',          (False, False,        True,    True)),
        ('      network inet raw,',     (False, False,        True,    True)),
        ('audit network,',              (False, False,        False,   False)),
        ('deny  network,',              (False, False,        False,   False)),
    )


class NetworkCoveredTest_05(NetworkCoveredTest):
    rule = 'deny network inet,'

    tests = (
        #   rule                      equal  strict equal  covered  covered exact
        ('      deny network inet,', (True,  True,         True,    True)),
        ('audit deny network inet,', (False, False,        False,   False)),
        ('           network inet,', (False, False,        False,   False)),  # XXX should covered be true here?
        ('      deny network unix,', (False, False,        False,   False)),
        ('      deny network,',      (False, False,        False,   False)),
    )


class NetworkCoveredTest_06(NetworkCoveredTest):
    rule = 'network (rw, connect) port=127 peer=(ip=192.168.122.3),'

    tests = (
        #   rule                                                                                     equal strict equal covered covered exact
        ('network (rw, connect) port=127 peer=(ip=192.168.122.3),',                                  (True,  True,       True,   True)),
        ('network (rw, connect) port=127 ip=192.168.122.2 peer=(ip=192.168.122.3),',                 (False, False,      True,   True)),
        ('network (rw, connect) inet port=127 ip=192.168.122.2 peer=(ip=192.168.122.3),',            (False, False,      True,   True)),
        ('network (rw, connect) port=127 ip=192.168.122.2 peer=(ip=192.168.122.3 port=12345),',      (False, False,      True,   True)),
        ('network (rw, connect) inet port=127 ip=192.168.122.2 peer=(ip=192.168.122.3 port=12345),', (False, False,      True,   True)),
        ('network connect port=12345 ip=192.168.122.2 peer=(ip=192.168.122.3),',                     (False, False,      False,  False)),
        ('network (r, connect) port=12345 ip=192.168.122.2 peer=(ip=192.168.122.3),',                (False, False,      False,  False)),
        ('network (r, connect) port=128 peer=(ip=192.168.122.3),',                                   (False, False,      False,  False)),
        ('network (rw, connect) port=127 peer=(ip=127.0.0.1),',                                      (False, False,      False,  False)),
        ('network (rw, connect) port=127,',                                                          (False, False,      False,  False)),
    )


class NetworkCoveredTest_Invalid(AATest):
    def test_borked_obj_is_covered_1(self):
        obj = NetworkRule.create_instance('network inet,')

        testobj = NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL)
        testobj.domain = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_borked_obj_is_covered_2(self):
        obj = NetworkRule.create_instance('network inet,')

        testobj = NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL)
        testobj.type_or_protocol = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_covered(self):
        raw_rule = 'network inet,'

        class SomeOtherClass(NetworkRule):
            pass

        obj = NetworkRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal(self):
        raw_rule = 'network inet,'

        class SomeOtherClass(NetworkRule):
            pass

        obj = NetworkRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class NetworkLogprofHeaderTest(AATest):
    tests = (
        ('network,',                                        [                              _('Accesses'), _('ALL'),      _('Network Family'), _('ALL'), _('Socket Type'), _('ALL'), _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),   # noqa: E201
        ('network inet,',                                   [                              _('Accesses'), _('ALL'),      _('Network Family'), 'inet',   _('Socket Type'), _('ALL'), _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),  # noqa: E201
        ('network inet stream,',                            [                              _('Accesses'), _('ALL'),      _('Network Family'), 'inet',   _('Socket Type'), 'stream', _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),  # noqa: E201
        ('deny network,',                                   [_('Qualifier'), 'deny',       _('Accesses'), _('ALL'),      _('Network Family'), _('ALL'), _('Socket Type'), _('ALL'), _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),
        ('allow network inet,',                             [_('Qualifier'), 'allow',      _('Accesses'), _('ALL'),      _('Network Family'), 'inet',   _('Socket Type'), _('ALL'), _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),
        ('audit network inet stream,',                      [_('Qualifier'), 'audit',      _('Accesses'), _('ALL'),      _('Network Family'), 'inet',   _('Socket Type'), 'stream', _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),
        ('audit deny network inet,',                        [_('Qualifier'), 'audit deny', _('Accesses'), _('ALL'),      _('Network Family'), 'inet',   _('Socket Type'), _('ALL'), _('Local'), _('ALL'),                    _('Peer'), _('ALL')]),
        ('network (bind, listen) stream ip=::1 port=22,',   [                              _('Accesses'), 'bind listen', _('Network Family'), _('ALL'), _('Socket Type'), 'stream', _('Local'), {'ip': '::1', 'port': '22'}, _('Peer'), _('ALL')]),  # noqa: E201
        ('audit deny network inet peer=(ip=::1),',          [_('Qualifier'), 'audit deny', _('Accesses'), _('ALL'),      _('Network Family'), 'inet',   _('Socket Type'), _('ALL'), _('Local'), _('ALL'),                    _('Peer'), {'ip': '::1'}]),
    )

    def _run_test(self, params, expected):
        obj = NetworkRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


class NetworkRuleReprTest(AATest):
    tests = (
        (NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL),                                       '<NetworkRule> network inet stream,'),
        (NetworkRule.create_instance(' allow  network  inet  stream, # foo'), '<NetworkRule> allow  network  inet  stream, # foo'),
    )

    def _run_test(self, params, expected):
        self.assertEqual(str(params), expected)


# --- tests for NetworkRuleset --- #
class NetworkRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = NetworkRuleset()
        ruleset_2 = NetworkRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

    def test_ruleset_1(self):
        ruleset = NetworkRuleset()
        rules = (
            'network tcp,',
            'network inet,',
        )

        expected_raw = [
            'network tcp,',
            'network inet,',
            '',
        ]

        expected_clean = [
            'network inet,',
            'network tcp,',
            '',
        ]

        for rule in rules:
            ruleset.add(NetworkRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())

    def test_ruleset_2(self):
        ruleset = NetworkRuleset()
        rules = (
            'network inet6 raw,',
            'allow network inet,',
            'deny network udp, # example comment',
        )

        expected_raw = [
            '  network inet6 raw,',
            '  allow network inet,',
            '  deny network udp, # example comment',
            '',
        ]

        expected_clean = [
            '  deny network udp, # example comment',
            '',
            '  allow network inet,',
            '  network inet6 raw,',
            '',
        ]

        for rule in rules:
            ruleset.add(NetworkRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw(1))
        self.assertEqual(expected_clean, ruleset.get_clean(1))


class NetworkGlobTestAATest(AATest):
    def setUp(self):
        self.maxDiff = None
        self.ruleset = NetworkRuleset()

    def test_glob_1(self):
        self.assertEqual(self.ruleset.get_glob('network inet,'), 'network,')

    # not supported or used yet
    # def test_glob_2(self):
    #     self.assertEqual(self.ruleset.get_glob('network inet raw,'), 'network inet,')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for network rules
            self.ruleset.get_glob_ext('network inet raw,')


class NetworkDeleteTestAATest(AATest):
    pass


class NetworkRulesetReprTest(AATest):
    def test_network_ruleset_repr(self):
        obj = NetworkRuleset()
        obj.add(NetworkRule(NetworkRule.ALL, 'inet', 'stream', NetworkRule.ALL, NetworkRule.ALL))
        obj.add(NetworkRule.create_instance(' allow  network  inet  stream, # foo'))

        expected = '<NetworkRuleset>\n  network inet stream,\n  allow  network  inet  stream, # foo\n</NetworkRuleset>'
        self.assertEqual(str(obj), expected)


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------

import unittest
from collections import namedtuple

from apparmor.common import AppArmorBug, AppArmorException
# from apparmor.logparser import ReadLog
from apparmor.rule.rlimit import RlimitRule, RlimitRuleset, split_unit
from apparmor.translations import init_translation
from common_test import AATest, setup_all_loops

_ = init_translation()

exp = namedtuple(
    'exp', ('audit', 'allow_keyword', 'deny', 'comment', 'rlimit', 'value', 'all_values'))

# --- tests for single RlimitRule --- #


class RlimitTest(AATest):
    def _compare_obj(self, obj, expected):
        self.assertEqual(expected.allow_keyword, obj.allow_keyword)
        self.assertEqual(expected.audit, obj.audit)
        self.assertEqual(expected.rlimit, obj.rlimit)
        self.assertEqual(expected.value, obj.value)
        self.assertEqual(expected.all_values, obj.all_values)
        self.assertEqual(expected.deny, obj.deny)
        self.assertEqual(expected.comment, obj.comment)


class RlimitTestParse(RlimitTest):
    tests = (
        # rawrule                                  audit  allow  deny   comment       rlimit        value        all/infinity?
        ('set rlimit as <= 2047MB,',           exp(False, False, False, '',           'as',         '2047MB',    False)),
        ('set rlimit as <= 2047 MB,',          exp(False, False, False, '',           'as',         '2047 MB',   False)),
        ('set rlimit cpu <= 1024,',            exp(False, False, False, '',           'cpu',        '1024',      False)),
        ('set rlimit stack <= 1024GB,',        exp(False, False, False, '',           'stack',      '1024GB',    False)),
        ('set rlimit stack <= 1024 GB,',       exp(False, False, False, '',           'stack',      '1024 GB',   False)),
        ('set rlimit rtprio <= 10, # comment', exp(False, False, False, ' # comment', 'rtprio',     '10',        False)),
        ('set rlimit core <= 44444KB,',        exp(False, False, False, '',           'core',       '44444KB',   False)),
        ('set rlimit core <= 44444 KB,',       exp(False, False, False, '',           'core',       '44444 KB',  False)),
        ('set rlimit rttime <= 60ms,',         exp(False, False, False, '',           'rttime',     '60ms',      False)),
        ('set rlimit cpu <= infinity,',        exp(False, False, False, '',           'cpu',        None,        True)),
        ('set rlimit nofile <= 256,',          exp(False, False, False, '',           'nofile',     '256',       False)),
        ('set rlimit data <= 4095KB,',         exp(False, False, False, '',           'data',       '4095KB',    False)),
        ('set rlimit cpu <= 12, # cmt',        exp(False, False, False, ' # cmt',     'cpu',        '12',        False)),
        ('set rlimit ofile <= 1234,',          exp(False, False, False, '',           'ofile',      '1234',      False)),
        ('set rlimit msgqueue <= 4444,',       exp(False, False, False, '',           'msgqueue',   '4444',      False)),
        ('set rlimit nice <= -10,',            exp(False, False, False, '',           'nice',       '-10',       False)),
        ('set rlimit rttime <= 60minutes,',    exp(False, False, False, '',           'rttime',     '60minutes', False)),
        ('set rlimit fsize <= 1023MB,',        exp(False, False, False, '',           'fsize',      '1023MB',    False)),
        ('set rlimit nproc <= 1,',             exp(False, False, False, '',           'nproc',      '1',         False)),
        ('set rlimit rss <= infinity, # cmt',  exp(False, False, False, ' # cmt',     'rss',        None,        True)),
        ('set rlimit memlock <= 10240,',       exp(False, False, False, '',           'memlock',    '10240',     False)),
        ('set rlimit sigpending <= 42,',       exp(False, False, False, '',           'sigpending', '42',        False)),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(RlimitRule.match(rawrule))
        obj = RlimitRule.create_instance(rawrule)
        self.assertEqual(rawrule.strip(), obj.raw_rule)
        self._compare_obj(obj, expected)


class RlimitTestParseInvalid(RlimitTest):
    tests = (
        ('set rlimit,',                    AppArmorException),  # missing parts
        ('set rlimit <= 5,',               AppArmorException),
        ('set rlimit cpu <= ,',            AppArmorException),
        ('set rlimit cpu <= "",',          AppArmorBug),        # evil quoting trick
        ('set rlimit foo <= 5,',           AppArmorException),  # unknown rlimit
        ('set rlimit rttime <= 60m,',      AppArmorException),  # 'm' could mean 'ms' or 'minutes'
        ('set rlimit cpu <= 20ms,',        AppArmorException),  # cpu doesn't support 'ms'...
        ('set rlimit cpu <= 20us,',        AppArmorException),  # ... or 'us'
        ('set rlimit nice <= 20MB,',       AppArmorException),  # invalid unit for this rlimit type
        ('set rlimit cpu <= 20MB,',        AppArmorException),
        ('set rlimit data <= 20seconds,',  AppArmorException),
        ('set rlimit locks <= 20seconds,', AppArmorException),
    )

    def _run_test(self, rawrule, expected):
        # self.assertFalse(RlimitRule.match(rawrule))  # the main regex isn't very strict
        with self.assertRaises(expected):
            RlimitRule.create_instance(rawrule)


class RlimitTestParseFromLog(RlimitTest):
    pass
    # def test_net_from_log(self):
    #     parser = ReadLog('', '', '')
    #
    #     event = 'type=AVC ...'
    #
    #     parsed_event = parser.parse_event(event)
    #
    #     self.assertEqual(parsed_event, {
    #         ...
    #     })
    #
    #     obj = RlimitRule(RlimitRule.ALL, parsed_event['name2'], log_event=parsed_event)
    #
    #     #              audit  allow  deny   comment  rlimit  value          all?
    #     expected = exp(False, False, False, '',      None,   '/foo/rename', False)
    #
    #     self._compare_obj(obj, expected)
    #
    #     self.assertEqual(obj.get_raw(1), '  rlimit -> /foo/rename,')


class RlimitFromInit(RlimitTest):
    tests = (
        # RlimitRule object                     audit  allow  deny   comment  rlimit     value     all/infinity?
        (RlimitRule('as', '2047MB'),        exp(False, False, False, '',      'as',     '2047MB',     False)),
        (RlimitRule('as', '2047 MB'),       exp(False, False, False, '',      'as',     '2047 MB',    False)),
        (RlimitRule('cpu', '1024'),         exp(False, False, False, '',      'cpu',    '1024',       False)),
        (RlimitRule('rttime', '60minutes'), exp(False, False, False, '',      'rttime',  '60minutes', False)),
        (RlimitRule('nice', '-10'),         exp(False, False, False, '',      'nice',   '-10',        False)),
        (RlimitRule('rss', RlimitRule.ALL), exp(False, False, False, '',      'rss',    None,         True)),
    )

    def _run_test(self, obj, expected):
        self._compare_obj(obj, expected)


class InvalidRlimitInit(AATest):
    tests = (
        # init params        expected exception
        (('as',   ''),       AppArmorBug),  # empty value
        (('',     '1024'),   AppArmorException),  # empty rlimit
        (('    ', '1024'),   AppArmorException),  # whitespace rlimit
        (('as',   '   '),    AppArmorBug),  # whitespace value
        (('xyxy', '1024'),   AppArmorException),  # invalid rlimit
        ((dict(), '1024'),   AppArmorBug),  # wrong type for rlimit
        ((None,   '1024'),   AppArmorBug),  # wrong type for rlimit
        (('as',   dict()),   AppArmorBug),  # wrong type for value
        (('as',   None),     AppArmorBug),  # wrong type for value
        (('cpu',  '100xy2'), AppArmorException),  # invalid unit
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            RlimitRule(*params)

    def test_missing_params_1(self):
        with self.assertRaises(TypeError):
            RlimitRule()

    def test_missing_params_2(self):
        with self.assertRaises(TypeError):
            RlimitRule('as')

    def test_allow_keyword(self):
        with self.assertRaises(AppArmorBug):
            RlimitRule('as', '1024MB', allow_keyword=True)

    def test_deny_keyword(self):
        with self.assertRaises(AppArmorBug):
            RlimitRule('as', '1024MB', deny=True)

    def test_audit_keyword(self):
        with self.assertRaises(AppArmorBug):
            RlimitRule('as', '1024MB', audit=True)

    def test_priority_keyword(self):
        with self.assertRaises(AppArmorBug):
            RlimitRule('as', '1024MB', priority=0)


class InvalidRlimitTest(AATest):
    def _check_invalid_rawrule(self, rawrule):
        obj = None
        self.assertFalse(RlimitRule.match(rawrule))
        with self.assertRaises(AppArmorException):
            obj = RlimitRule.create_instance(rawrule)

        self.assertIsNone(obj, 'RlimitRule handed back an object unexpectedly')

    def test_invalid_net_missing_comma(self):
        self._check_invalid_rawrule('rlimit')  # missing comma

    def test_invalid_net_non_RlimitRule(self):
        self._check_invalid_rawrule('dbus,')  # not a rlimit rule

    def test_empty_net_data_1(self):
        obj = RlimitRule('as', '1024MB')
        obj.rlimit = ''
        # no rlimit set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)

    def test_empty_net_data_2(self):
        obj = RlimitRule('as', '1024MB')
        obj.value = ''
        # no value set, and ALL not set
        with self.assertRaises(AppArmorBug):
            obj.get_clean(1)


class WriteRlimitTest(AATest):
    tests = (
        #  raw rule                                                  clean rule
        ('     set rlimit    cpu   <= 1024     ,    # foo        ', 'set rlimit cpu <= 1024, # foo'),
        ('        set     rlimit  stack <= 1024GB  ,',              'set rlimit stack <= 1024GB,'),
        ('   set rlimit    rttime <= 100ms   ,   # foo bar',        'set rlimit rttime <= 100ms, # foo bar'),
        ('   set rlimit      cpu   <= infinity   ,  ',              'set rlimit cpu <= infinity,'),
        ('    set rlimit     msgqueue <=   4444 ,   ',              'set rlimit msgqueue <= 4444,'),
        ('    set rlimit   nice    <=  5   ,   # foo bar',          'set rlimit nice <= 5, # foo bar'),
        ('    set rlimit   nice <=   -5    , # cmt',                'set rlimit nice <= -5, # cmt'),
    )

    def _run_test(self, rawrule, expected):
        self.assertTrue(RlimitRule.match(rawrule))
        obj = RlimitRule.create_instance(rawrule)
        clean = obj.get_clean()
        raw = obj.get_raw()

        self.assertEqual(expected.strip(), clean, 'unexpected clean rule')
        self.assertEqual(rawrule.strip(), raw, 'unexpected raw rule')

    def test_write_manually(self):
        obj = RlimitRule('as', '1024MB')

        expected = '    set rlimit as <= 1024MB,'

        self.assertEqual(expected, obj.get_clean(2), 'unexpected clean rule')
        self.assertEqual(expected, obj.get_raw(2), 'unexpected raw rule')


class RlimitCoveredTest(AATest):
    def _run_test(self, param, expected):
        obj = RlimitRule.create_instance(self.rule)
        check_obj = RlimitRule.create_instance(param)

        self.assertTrue(RlimitRule.match(param))

        self.assertEqual(
            obj.is_equal(check_obj), expected[0],
            'Mismatch in is_equal, expected {}'.format(expected[0]))
        self.assertEqual(
            obj.is_equal(check_obj, True), expected[1],
            'Mismatch in is_equal/strict, expected {}'.format(expected[1]))

        self.assertEqual(
            obj.is_covered(check_obj), expected[2],
            'Mismatch in is_covered, expected {}'.format(expected[2]))
        self.assertEqual(
            obj.is_covered(check_obj, True, True), expected[3],
            'Mismatch in is_covered/exact, expected {}'.format(expected[3]))


class RlimitCoveredTest_01(RlimitCoveredTest):
    rule = 'set rlimit cpu <= 150,'

    tests = (
        #   rule                           equal  strict equal  covered  covered exact
        ('set rlimit as <= 100MB,',       (False, False,        False,   False)),
        ('set rlimit rttime <= 150,',     (False, False,        False,   False)),
        ('set rlimit cpu <= 100,',        (False, False,        True,    True)),
        ('set rlimit cpu <= 150,',        (True,  True,         True,    True)),
        ('set rlimit cpu <= 300,',        (False, False,        False,   False)),
        ('set rlimit cpu <= 10seconds,',  (False, False,        True,    True)),
        ('set rlimit cpu <= 150seconds,', (True,  False,        True,    True)),
        ('set rlimit cpu <= 300seconds,', (False, False,        False,   False)),
        ('set rlimit cpu <= 1minutes,',   (False, False,        True,    True)),
        ('set rlimit cpu <= 1min,',       (False, False,        True,    True)),
        ('set rlimit cpu <= 3minutes,',   (False, False,        False,   False)),
        ('set rlimit cpu <= 1hour,',      (False, False,        False,   False)),
        ('set rlimit cpu <= 2 days,',     (False, False,        False,   False)),
        ('set rlimit cpu <= 1 week,',     (False, False,        False,   False)),
    )


class RlimitCoveredTest_02(RlimitCoveredTest):
    rule = 'set rlimit data <= 4MB,'

    tests = (
        #   rule                         equal  strict equal  covered  covered exact
        ('set rlimit data <= 100,',     (False, False,        True,    True)),
        ('set rlimit data <= 2KB,',     (False, False,        True,    True)),
        ('set rlimit data <= 2MB,',     (False, False,        True,    True)),
        ('set rlimit data <= 4194304,', (True,  False,        True,    True)),
        ('set rlimit data <= 4096KB,',  (True,  False,        True,    True)),
        ('set rlimit data <= 4MB,',     (True,  True,         True,    True)),
        ('set rlimit data <= 4 MB,',    (True,  False,        True,    True)),
        ('set rlimit data <= 6MB,',     (False, False,        False,   False)),
        ('set rlimit data <= 6 MB,',    (False, False,        False,   False)),
        ('set rlimit data <= 1GB,',     (False, False,        False,   False)),
    )


class RlimitCoveredTest_03(RlimitCoveredTest):
    rule = 'set rlimit nice <= -1,'

    tests = (
        #   rule                    equal  strict equal  covered  covered exact
        ('set rlimit nice <= 5,',  (False, False,        True,    True)),
        ('set rlimit nice <= 0,',  (False, False,        True,    True)),
        ('set rlimit nice <= -1,', (True,  True,         True,    True)),
        ('set rlimit nice <= -3,', (False, False,        False,   False)),
    )


class RlimitCoveredTest_04(RlimitCoveredTest):
    rule = 'set rlimit locks <= 42,'

    tests = (
        #   rule                           equal  strict equal  covered  covered exact
        ('set rlimit locks <= 20,',       (False, False,        True,    True)),
        ('set rlimit locks <= 40,',       (False, False,        True,    True)),
        ('set rlimit locks <= 42,',       (True,  True,         True,    True)),
        ('set rlimit locks <= 60,',       (False, False,        False,   False)),
        ('set rlimit locks <= infinity,', (False, False,        False,   False)),
    )


class RlimitCoveredTest_05(RlimitCoveredTest):
    rule = 'set rlimit locks <= infinity,'

    tests = (
        #   rule                           equal  strict equal  covered  covered exact
        ('set rlimit locks <= 20,',       (False, False,        True,    True)),
        ('set rlimit cpu <= 40,',         (False, False,        False,   False)),
        ('set rlimit locks <= infinity,', (True,  True,         True,    True)),
    )


class RlimitCoveredTest_Invalid(AATest):
    def test_borked_obj_is_covered_1(self):
        obj = RlimitRule.create_instance('set rlimit cpu <= 1024,')

        testobj = RlimitRule('cpu', '1024')
        testobj.rlimit = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_borked_obj_is_covered_2(self):
        obj = RlimitRule.create_instance('set rlimit cpu <= 1024,')

        testobj = RlimitRule('cpu', '1024')
        testobj.value = ''

        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_covered(self):
        raw_rule = 'set rlimit cpu <= 1024,'

        class SomeOtherClass(RlimitRule):
            pass

        obj = RlimitRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_covered(testobj)

    def test_invalid_is_equal(self):
        raw_rule = 'set rlimit cpu <= 1024,'

        class SomeOtherClass(RlimitRule):
            pass

        obj = RlimitRule.create_instance(raw_rule)
        testobj = SomeOtherClass.create_instance(raw_rule)  # different type
        with self.assertRaises(AppArmorBug):
            obj.is_equal(testobj)


class RlimitLogprofHeaderTest(AATest):
    tests = (
        ('set rlimit cpu <= infinity,', [_('Rlimit'), 'cpu',    _('Value'), 'infinity']),
        ('set rlimit as <= 200MB,',     [_('Rlimit'), 'as',     _('Value'), '200MB']),
        ('set rlimit rttime <= 200ms,', [_('Rlimit'), 'rttime', _('Value'), '200ms']),
        ('set rlimit nproc <= 1,',      [_('Rlimit'), 'nproc',  _('Value'), '1']),
    )

    def _run_test(self, params, expected):
        obj = RlimitRule.create_instance(params)
        self.assertEqual(obj.logprof_header(), expected)


# --- tests for RlimitRuleset --- #

class RlimitRulesTest(AATest):
    def test_empty_ruleset(self):
        ruleset = RlimitRuleset()
        ruleset_2 = RlimitRuleset()
        self.assertEqual([], ruleset.get_raw(2))
        self.assertEqual([], ruleset.get_clean(2))
        self.assertEqual([], ruleset_2.get_raw(2))
        self.assertEqual([], ruleset_2.get_clean(2))

    def test_ruleset_1(self):
        ruleset = RlimitRuleset()
        rules = [
            '  set rlimit cpu  <= 100,',
            '  set rlimit as   <= 50MB,',
        ]

        expected_raw = [
            'set rlimit cpu  <= 100,',
            'set rlimit as   <= 50MB,',
            '',
        ]

        expected_clean = [
            'set rlimit as <= 50MB,',
            'set rlimit cpu <= 100,',
            '',
        ]

        for rule in rules:
            ruleset.add(RlimitRule.create_instance(rule))

        self.assertEqual(expected_raw, ruleset.get_raw())
        self.assertEqual(expected_clean, ruleset.get_clean())


class RlimitGlobTestAATest(AATest):
    def setUp(self):
        self.ruleset = RlimitRuleset()

    def test_glob_1(self):
        with self.assertRaises(AppArmorBug):
            self.ruleset.get_glob('set rlimit cpu <= 100,')

    # not supported or used yet, glob behaviour not decided yet
    # def test_glob_2(self):
    #     self.assertEqual(self.ruleset.get_glob('rlimit /foo -> /bar,'), 'rlimit -> /bar,')

    def test_glob_ext(self):
        with self.assertRaises(NotImplementedError):
            # get_glob_ext is not available for rlimit rules
            self.ruleset.get_glob_ext('set rlimit cpu <= 100,')


class RlimitDeleteTestAATest(AATest):
    # no de-duplication tests for rlimit - de-duplication consists of is_covered() (which we already test) and
    # code from BaseRuleset (which is already tested in the capability, network and change_profile tests).
    # Therefore no need to test it once more ;-)
    pass


# --- other tests --- #
class RlimitSplit_unitTest(AATest):
    tests = (
        ('40MB',  (40, 'MB')),
        ('40 MB', (40, 'MB')),
        ('40',    (40, '')),
    )

    def _run_test(self, params, expected):
        self.assertEqual(split_unit(params), expected)

    def test_invalid_split_unit(self):
        with self.assertRaises(AppArmorBug):
            split_unit('MB')


class RlimitSize_to_intTest(AATest):
    def AASetup(self):
        self.obj = RlimitRule('cpu', '1')

    tests = (
        ('40GB', 40 * 1024 * 1024 * 1024),
        ('40MB', 41943040),
        ('40KB', 40960),
        ('40',   40),
    )

    def _run_test(self, params, expected):
        self.assertEqual(self.obj.size_to_int(params), expected)

    def test_invalid_size_to_int_01(self):
        with self.assertRaises(AppArmorException):
            self.obj.size_to_int('20mice')


class RlimitTime_to_intTest(AATest):
    def AASetup(self):
        self.obj = RlimitRule('cpu', '1')

    tests = (
        ('40',        0.00004),
        ('30us',      0.00003),
        ('40ms',      0.04),
        ('40seconds', 40),
        ('2minutes',  2 * 60),
        ('2hours',    2 * 60 * 60),
        ('1 day',     1 * 60 * 60 * 24),
        ('2 weeks',   2 * 60 * 60 * 24 * 7),
    )

    def _run_test(self, params, expected):
        self.assertEqual(self.obj.time_to_int(params, 'us'), expected)

    def test_with_seconds_as_default(self):
        self.assertEqual(self.obj.time_to_int('40', 'seconds'), 40)

    def test_with_ms_as_default(self):
        self.assertEqual(self.obj.time_to_int('40', 'ms'), 0.04)

    def test_with_us_as_default(self):
        self.assertEqual(self.obj.time_to_int('30', 'us'), 0.00003)

    def test_invalid_time_to_int(self):
        with self.assertRaises(AppArmorException):
            self.obj.time_to_int('20mice', 'seconds')


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)



# ----------------------------------------------------------------------
#    Copyright (c) 2012 Canonical Ltd.
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, contact Canonical, Ltd.
# ----------------------------------------------------------------------
#
# Usage:
# $ python ./python-tools-setup.py install --root=... --version=...
#
# Note: --version=... must be the last argument to this script
#

import os
import shutil
import sys

from setuptools import setup
from setuptools.command.install import install as _install

# removeprefix is only in python 3.9+ support older python versions
def replace_path_prefix(text, prefix):
    if text.startswith(prefix):
        suffix = text[len(prefix):]
        if not suffix.startswith("/"):
            suffix = "/" + suffix
        return suffix
    return text

class Install(_install):
    """Override setuptools to install the files where we want them."""
    def run(self):
        # Now byte-compile everything
        super().run()

        prefix = self.prefix
        if self.root is not None:
            prefix = self.root

        # Install scripts, configuration files and data
        scripts = ('/usr/bin/aa-easyprof',)
        self.mkpath(prefix + os.path.dirname(scripts[0]))
        for s in scripts:
            f = prefix + s
            self.copy_file(os.path.basename(s), f)

        configs = ('easyprof/easyprof.conf',)
        self.mkpath(prefix + "/etc/apparmor")
        for c in configs:
            self.copy_file(c, os.path.join(prefix + "/etc/apparmor", os.path.basename(c)))

        data = ('easyprof/templates', 'easyprof/policygroups')
        self.mkpath(prefix + "/usr/share/apparmor/easyprof")
        for d in data:
            self.copy_tree(d, os.path.join(prefix + "/usr/share/apparmor/easyprof", os.path.basename(d)))

        # Make update_profile.py executable
        update_profile_path = os.path.join(self.install_lib, 'apparmor/update_profile.py')
        print('changing mode of {} to 755'.format(update_profile_path))
        os.chmod(update_profile_path, 0o755)

        pkexec_action_name = 'net.apparmor.pkexec.aa-notify.policy'
        print('Installing {} to /usr/share/polkit-1/actions/ mode 644'.format(pkexec_action_name))
        with open(pkexec_action_name, 'r') as f:
            polkit_template = f.read()

        # don't leak the buildroot into the polkit files
        polkit = polkit_template.format(LIB_PATH=replace_path_prefix(self.install_lib, prefix))

        if not os.path.exists(prefix + '/usr/share/polkit-1/actions/'):
            self.mkpath(prefix + '/usr/share/polkit-1/actions/')
        with open(prefix + '/usr/share/polkit-1/actions/' + pkexec_action_name, 'w') as f:
            f.write(polkit)
        os.chmod(prefix + '/usr/share/polkit-1/actions/' + pkexec_action_name, 0o644)


if os.path.exists('staging'):
    shutil.rmtree('staging')
shutil.copytree('apparmor', 'staging')

# Support the --version=... since this will be part of a Makefile
version = "unknown-version"
if "--version=" in sys.argv[-1]:
    version = sys.argv[-1].split('=')[1].replace('~', '-')
    sys.argv = sys.argv[0:-1]

setup(
    name='apparmor',
    version=version,
    description='Python libraries for AppArmor utilities',
    long_description='Python libraries for AppArmor utilities',
    author='AppArmor Developers',
    author_email='apparmor@lists.ubuntu.com',
    url='https://gitlab.com/apparmor/apparmor',
    license='GPL-2',
    cmdclass={'install': Install},
    package_dir={'apparmor': 'staging'},
    packages=['apparmor', 'apparmor.rule'],
    py_modules=['apparmor.easyprof']
)

shutil.rmtree('staging')



#!/usr/bin/python3
# ----------------------------------------------------------------------
#    Copyright (C) 2013 Kshitij Gupta <kgupta8592@gmail.com>
#    Copyright (C) 2014 Canonical, Ltd.
#    Copyright (C) 2015 Christian Boltz <apparmor@cboltz.de>
#
#    This program is free software; you can redistribute it and/or
#    modify it under the terms of version 2 of the GNU General Public
#    License as published by the Free Software Foundation.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
# ----------------------------------------------------------------------
import unittest

import apparmor.severity as severity
from apparmor.common import AppArmorException
from common_test import AATest, setup_all_loops


class SeverityBaseTest(AATest):

    def AASetup(self):
        self.sev_db = severity.Severity('../severity.db', 'unknown')

    def _capability_severity_test(self, cap, expected_rank):
        rank = self.sev_db.rank_capability(cap)
        self.assertEqual(rank, expected_rank,
                         'expected rank {}, got {}'.format(expected_rank, rank))

    def _simple_severity_w_perm(self, path, perm, expected_rank):
        rank = self.sev_db.rank_path(path, perm)
        self.assertEqual(rank, expected_rank,
                         'expected rank {}, got {}'.format(expected_rank, rank))


class SeverityTest(SeverityBaseTest):
    tests = (
        (('/usr/bin/whatis',          'x'),   5),
        (('/etc',                     'x'),   'unknown'),
        (('/dev/doublehit',           'x'),   0),
        (('/dev/doublehit',           'rx'),  4),
        (('/dev/doublehit',           'rwx'), 8),
        (('/dev/tty10',               'rwx'), 9),
        (('/var/adm/foo/**',          'rx'),  3),
        (('/etc/apparmor/**',         'r'),   6),
        (('/etc/**',                  'r'),   'unknown'),
        (('/usr/foo@bar',             'r'),   'unknown'),  # filename containing @
        (('/home/foo@bar',            'rw'),  6),  # filename containing @
        (('/etc/apache2/ssl.key/bar', 'r'),   7),  # /etc/apache2/** (3) vs. /etc/apache2/**ssl** (7)
        (('/etc/apache2/foo/ssl/bar', 'r'),   7),  # additional path level triggers otherwise untested branch
        (('/proc/sys/kernel/hotplug', 'rwx'), 10),  # non-glob filename, severity depends on mode
    )

    def _run_test(self, params, expected):
        self._simple_severity_w_perm(params[0], params[1], expected)

    def test_invalid_rank(self):
        with self.assertRaises(AppArmorException):
            self._simple_severity_w_perm('unexpected_unput', 'rw', 6)


class SeverityTestCap(SeverityBaseTest):
    tests = (
        ('KILL', 8),
        ('SETPCAP', 9),
        ('setpcap', 9),
        ('UNKNOWN', 'unknown'),
        ('K*', 'unknown'),
        ('__ALL__', 10),
    )

    def _run_test(self, params, expected):
        self._capability_severity_test(params, expected)

        rank = self.sev_db.rank_capability(params)
        self.assertEqual(rank, expected, 'expected rank {}, got {}'.format(expected, rank))


class SeverityVarsTest(SeverityBaseTest):
    tests = (
        (('@{PROC}/sys/vm/overcommit_memory',          'r'), 6),
        (('@{HOME}/sys/@{PROC}/overcommit_memory',     'r'), 4),
        (('/overco@{multiarch}mmit_memory',            'r'), 'unknown'),
        (('@{PROC}/sys/@{TFTP_DIR}/overcommit_memory', 'r'), 6),
        (('@{somepaths}/somefile',                     'r'), 7),
        (('@{strangevar}/somefile',                    'r'), 6),
    )

    def _run_test(self, params, expected):
        vars = {
            # Note: using [] instead of {} is intentional to keep order of checking (and therefore code coverage) constant
            '@{HOME}':       ['@{HOMEDIRS}/*/', '/root/'],
            '@{HOMEDIRS}':   ['/home/', '/storage/'],
            '@{multiarch}':  ['*-linux-gnu*'],
            '@{TFTP_DIR}':   ['/var/tftp /srv/tftpboot'],
            '@{PROC}':       ['/proc/'],
            '@{somepaths}':  ['/home/foo/downloads', '@{HOMEDIRS}/foo/.ssh/'],
            '@{strangevar}': ['/srv/', '/proc/'],
        }
        self.sev_db.set_variables(vars)
        self._simple_severity_w_perm(params[0], params[1], expected)


class SeverityDBTest(AATest):
    def _test_db(self, contents):
        self.db_file = self.writeTmpfile('severity.db', contents)
        self.sev_db = severity.Severity(self.db_file)
        return self.sev_db

    tests = (
        ("CAP_LEASE 18\n",               AppArmorException),  # out of range
        ("CAP_LEASE -1\n",               AppArmorException),  # out of range
        ("/etc/passwd* 0 4\n",           AppArmorException),  # insufficient vals
        ("/etc/passwd* 0 4 5 6\n",       AppArmorException),  # too many vals
        ("/etc/passwd* -2 4 6\n",        AppArmorException),  # out of range
        ("/etc/passwd* 12 4 6\n",        AppArmorException),  # out of range
        ("/etc/passwd* 2 -4 6\n",        AppArmorException),  # out of range
        ("/etc/passwd 2 14 6\n",         AppArmorException),  # out of range
        ("/etc/passwd 2 4 -12\n",        AppArmorException),  # out of range
        ("/etc/passwd 2 4 4294967297\n", AppArmorException),  # out of range
        ("garbage line\n",               AppArmorException),
    )

    def _run_test(self, params, expected):
        with self.assertRaises(expected):
            self._test_db(params)

    def test_simple_db(self):
        self._test_db('''
    CAP_LEASE 8
    /etc/passwd*    4 8 0
''')

    def test_cap_val_max_range(self):
        self._test_db("CAP_LEASE 10\n")

    def test_cap_val_min_range(self):
        self._test_db("CAP_LEASE 0\n")

    def test_invalid_db(self):
        self.assertRaises(AppArmorException, severity.Severity, 'severity_broken.db')

    def test_nonexistent_db(self):
        self.assertRaises(IOError, severity.Severity, 'severity.db.does.not.exist')

    def test_no_arg_to_severity(self):
        with self.assertRaises(AppArmorException):
            severity.Severity()


setup_all_loops(__name__)
if __name__ == '__main__':
    unittest.main(verbosity=1)
